/*
Licensed Materials - Property of IBM
5725-C95
(C) Copyright IBM Corporation 2012. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

define(["dojo/_base/array", "./BPM_CONSTANTS", "./CONSTANTS", "./commonHelper", "./coachViewHelper", "./dynamicBindingHelper", "./engine"], function(dojoArray, BPM_CONSTANTS, CONSTANTS, commonHelper, coachViewHelper, dynamicBindingHelper, engine) {

	return function () {

		this._bindingHandlers = [];
		
		this.load = function() {
			commonHelper.timestamp("Enter Coach.load()");

			/* sample content of this._visibilityConfigs produced by code-gen
				[
					{
						viewInstancePath: ["Text2"],
						variablesToWatch: ["local.var1", "local.var3"],
						getVisibility: function(context, event, local) {
							return bpmConstants.VISIBILITY_DEFAULT;
						}
					}
				];	
			*/	

			if(!this._visibilityConfigs) {
				commonHelper.timestamp("Exit Coach.load(), _visibilityConfigs doesn't exist");
				return;
			}
			
			for(var i=0; i<this._visibilityConfigs.length; i++) {
				var aConfig = this._visibilityConfigs[i];
				// aConfig works as the scope object for bind(), when callback is invoked, need to access coach object
				aConfig._coach = this;
				// cache string version of viewInstancePath for later usage (mainly for comparison)
				aConfig.viewInstancePathStr = this._viewPathToString(aConfig.viewInstancePath);
				
				aConfig.variablesToWatch = this._preprocessWatchedVariables(aConfig.variablesToWatch);
				
				for(var j=0; j<aConfig.variablesToWatch.length; j++) {
					var callbackToManageVisibility = function(changeEvent) {
							// don't modify changeEvent which is supposed to be readonly
							// changeEvent coming from dynamicBindingHelper contains some internal information that shouldn't be exposed.
							var event = {
								type: BPM_CONSTANTS.VISIBILITY_SCRIPT_EVENT_TYPE_CHANGE,
								property: changeEvent.property,
								newVal: changeEvent.newVal,
								oldVal: changeEvent.oldVal,
								path:	changeEvent.path
							};
							if(typeof changeEvent._origin != "undefined") {
								event._origin = changeEvent._origin;
							};

							// "this" is visibilityConfig object
							this._coach._updateViewsVisibility(this, event);
						};					

					var bindingHandler = dynamicBindingHelper.bind(aConfig.variablesToWatch[j], callbackToManageVisibility, aConfig);
					this._bindingHandlers.push(bindingHandler);
				}
			}
			commonHelper.timestamp("Exit Coach.load()");
		}
		
		/*
		 	- if the watched variable is "local.abc[]", also watch "local.abc[index]"
		 	- if the watched variable is "local.abc[index]", also watch "local.abc[]"
		 	- filter out the duplicated variables in the array 
		*/
		this._preprocessWatchedVariables = function(watachedVariableArray) {
			var refinedArray = [];
			var oneVariable;
			for(var k=0; k<watachedVariableArray.length; k++) {
				oneVariable = watachedVariableArray[k];
				var pathArray = oneVariable.split(".");
				var lastPath = pathArray[pathArray.length - 1];
				var extraVariable = null;
				if(lastPath.indexOf(CONSTANTS.INDEX_WITH_BRACKET) != -1) {
					pathArray[pathArray.length - 1] = lastPath.replace(CONSTANTS.INDEX_WITH_BRACKET, "[]");
					extraVariable = pathArray.join(".");					
				} else if(lastPath.indexOf("[]") != -1) {
					pathArray[pathArray.length - 1] = lastPath.replace("[]", CONSTANTS.INDEX_WITH_BRACKET);
					extraVariable = pathArray.join(".");					
				}
								
				if(dojoArray.indexOf(refinedArray, oneVariable) == -1) {
					// new variable
					refinedArray.push(oneVariable);
				}
				if(!!extraVariable && dojoArray.indexOf(refinedArray, extraVariable) == -1) {
					// new variable
					refinedArray.push(extraVariable);
				}
			}
			return refinedArray;
		}
		
		this.unload = function() {
			// unregister what we have binded
			try {
				dynamicBindingHelper.unbindMultiple(this._bindingHandlers);
			} catch(ex) {
		    	commonHelper.consoleLog("Coach.unload(), failed to unbind the binding handler, Exception: " + ex, commonHelper.ERROR);
			}
		}

		this.view = function() {
			commonHelper.timestamp("Enter Coach.view()");
			if(!!this._visibilityConfigs) {
				for(var i=0; i<this._visibilityConfigs.length; i++) {
					var aConfig = this._visibilityConfigs[i];
					var event = {type: BPM_CONSTANTS.VISIBILITY_SCRIPT_EVENT_TYPE_INIT};
					this._updateViewsVisibility(aConfig, event);
				}			
			}
			commonHelper.timestamp("Exit Coach.view()");
		}
		
		this.updateVisibilityIfApplicable = function(coachView) {
			if(!this._visibilityConfigs) {
				// no coach level visibility configure, need to do nothing
				return;
			}
			
			// coachView._isVisibilityInitialized should be only set and get in this function
			// here is the reason to introduce coachView._isVisibilityInitialized: view managed ones and framework managed ones may mix in the view hierarchy.
			// as the result, this.updateVisibilityIfApplicable() might be called in one coachView more than once  
			if(coachView._isVisibilityInitialized) {
				// coachView has already been handled
				return;
			}
			
			// handle subviews firstly
			if (!!coachView.context.subview) {
				for (var viewId in coachView.context.subview) {
					var subviews = coachView.context.subview[viewId];
					for(var divId in subviews) {
						this.updateVisibilityIfApplicable( subviews[divId]);
					}
				}		
			}
			
			// now handle coachView itself
			var viewInstancePath = coachViewHelper.getViewPath(coachView);
			var viewInstancePathStr = this._viewPathToString(viewInstancePath);
			for(var i=0; i<this._visibilityConfigs.length; i++) {
				if(this._visibilityConfigs[i].viewInstancePathStr == viewInstancePathStr) {
					var event = {type: BPM_CONSTANTS.VISIBILITY_SCRIPT_EVENT_TYPE_INIT};
					var newVisibility = this._computeVisibility(this._visibilityConfigs[i], event);
					if(!!newVisibility) {
						this._setVisibility(coachView, newVisibility);
					}
					return;
				}
			}			

			// coachView._isVisibilityInitialized should be only set and get in this function
			coachView._isVisibilityInitialized = true;			
		}
		
		this._viewPathToString = function(viewPath) {
			return viewPath.join(".");
		}
		
		this._computeVisibility = function(aVisibilityConfig, event) {
			var getterFunc = aVisibilityConfig.getVisibility;
			// context
			var context = {bpm: {system: engine.GlobalAssets.system, team: engine.GlobalAssets.team}};	
			var parameters = [context, event, engine.GlobalAssets.local];

			try {		
				var startTime = commonHelper.isDebugMode() ? (new Date()).getTime() : null;
				var newVisibility = getterFunc.apply(null, parameters);
				if(startTime) {
					var elapsedTime = (new Date()).getTime() - startTime;
					if(elapsedTime >= CONSTANTS.ELAPSED_TIME_THRESHOLD_VISIBILITY_SCRIPT) {
						var info = this._getScriptDisplayName(aVisibilityConfig.viewInstancePath) + " used " + elapsedTime + "ms";
						commonHelper.timestamp(info);
					}
				}
				return newVisibility;
		    } catch(ex) {
		    	try {
			    	var message = "Coach framework caught exception when calling " + this._getScriptDisplayName(aVisibilityConfig.viewInstancePath);
			    	message +=  ", Exception: " + ex;
			    	commonHelper.consoleLog(message, commonHelper.ERROR);
		    	} catch(ex1) {
			    	commonHelper.consoleLog("Even failed to gracefully log this Exception: " + ex, commonHelper.ERROR);
		    	}
		    	
		    	// failed to get the visibility, make no sense to continue
		    	return null;
		    }
		}
		
		this._setVisibility = function(coachView, newVisibility) {
			if(coachView.context.options._metadata.visibility.get("value") != newVisibility) {
				coachView.context.options._metadata.visibility.set("value", newVisibility);
			}
		}
		
		this._isValidVisibility = function(visibility) {
			return (visibility && 
				(visibility == BPM_CONSTANTS.VISIBILITY_DEFAULT ||
				 visibility == BPM_CONSTANTS.VISIBILITY_REQUIRED ||
				 visibility == BPM_CONSTANTS.VISIBILITY_EDITABLE ||
				 visibility == BPM_CONSTANTS.VISIBILITY_READONLY ||
				 visibility == BPM_CONSTANTS.VISIBILITY_NONE ||
				 visibility == BPM_CONSTANTS.VISIBILITY_HIDDEN));
		}
		
		this._updateViewsVisibility = function(aVisibilityConfig, event) {
			var newVisibility = this._computeVisibility(aVisibilityConfig, event);
			if(!this._isValidVisibility(newVisibility)) {
				// not valid visibility, log and ignore it				
		    	var message = "Coach framework ignores an invalid visibility [" + newVisibility + "] after calling " + this._getScriptDisplayName(aVisibilityConfig.viewInstancePath);
		    	commonHelper.consoleLog(message, commonHelper.WARN);
				return;
			}
			
			// locates the viewInstances based on viewInstancePath
			var viewInstances = coachViewHelper.findAllViewInstances(engine.GlobalAssets.topLevelViewScope, aVisibilityConfig.viewInstancePath);		
			for(var	k=0; k<viewInstances.length; k++) {
				this._setVisibility(viewInstances[k], newVisibility);
			}
		}

		this._getScriptDisplayName = function(viewInstancePath) {
			return "Visibility script for view(s) represented by path [" + viewInstancePath.join() + "]";
		}
		
	};
});