/*
Licensed Materials - Property of IBM
5725-C95
(C) Copyright IBM Corporation 2012, 2014. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

// Contains all common utilities

define(["dojo/_base/array", "./CONSTANTS", "./dataBindingHelper"], function(dojoArray, CONSTANTS, dataBindingHelper) {
				
	var coachUtil = new Object();

	/*
	 returns true if the array contains the item, otherwise returns false.
	 Note: update the implementation to array.indexOf() once IE8 is not supported.
	*/
	coachUtil.contains = function(array, item){
    	return dojoArray.indexOf(array, item) >= 0;
  	};

  	/*
  	 dateArray is an array of Date objects, dateInMilliseconds is the number of milliseconds since midnight Jan 1, 1970
  	 returns true if one date object in the dateArray match dateInMilliseconds, otherwise returns false.
  	*/
  	coachUtil.isDateMatched = function(dateArray, dateInMilliseconds) {
  		if(!!dateArray) {
  			for(var k=0; k<dateArray.length; k++) {
  				if(dateArray[k].getTime && dateArray[k].getTime() == dateInMilliseconds) {
  					return true;
  				}
  			}
  		}
  		return false;	
  	};
  	
  	/*
  	 returns true if one item in the list has "id" property which is equal to the specified id parameter, otherwise returns false.
  	*/
  	coachUtil.isIdMatched = function(array, id) {
  		if(!!array) {
  			for(var k=0; k<array.length; k++) {
  				if(array[k].id == id) {
  					return true;
  				}
  			}
  		}
  		return false;	
  	};
  	
  	/*
  	 bindingStr may represent many variables that are under root variable local, e.g. bindingStr is "local.contactBook.contacts[index].phones[index].number".
  	 This API tests whether some variable (represented by bindingStr and local) passes the test implemented by the provided callback.
  	 	
  	 	bindingStr: examples are "local.contactBook.owner", "local.contactBook.contacts.listSelected.phones[index].name"
  	 	local: the root Statful object that holds all human service variables used by Coach
  	 	callback: function to test for each variable
  	 	thisObject: OBject to use as this when executing callback
  	*/
  	coachUtil.some = function(bindingStr, local, callback, thisObject) {
  		if(!bindingStr) {
  			// bindingStr shouldn't be null or undefined
  			return false;
  		}
  		
		var sourcePathArray = bindingStr.split(".");
		if(sourcePathArray.length < 2 || sourcePathArray[0] != "local") {
			// bindingStr should always start with "local." and contains at least 2 paths
			return false;
		}
		
		var pathArray = [];
		var onePath;
		for(var k=0; k<sourcePathArray.length; k++) {
			onePath = sourcePathArray[k];			
			if(onePath.indexOf(CONSTANTS.INDEX_WITH_BRACKET) != -1) {
				// e.g. a[currentItem]
				var virtualPathForList = onePath.replace(CONSTANTS.INDEX_WITH_BRACKET, "");
				pathArray.push(virtualPathForList);
				pathArray.push(CONSTANTS.INDEX_WITH_BRACKET);				
			} else {			
				pathArray.push(onePath);
			}
		}
		
		// if bindingStr is "local.contactBook.contacts[index].phones[index].number", 
		// pathArray is ["local", "contactBook", "contacts", "[index]", "phones", "[index]", "number"]
		
		return coachUtil._traverse(pathArray, 1, local, callback, thisObject);
  	};
  	
	coachUtil._traverse = function(pathArray, index, dataObject, compareFunc, thisObject) {
		if(!dataObject) {
			// for example, pathArray is ["local", "contactBook", "contacts", "listSelected", "phones"]
			// if no selection on "local.contactBook.contacts" list, the dataObject for "local.contactBook.contacts.listSelected.phones" is undefined
			return false;
		} 
	
		var path = pathArray[index];
		var isLastPath = (index == pathArray.length - 1);
		
		if(path == CONSTANTS.INDEX_WITH_BRACKET) {
			// dataObject represents a com.ibm.bpm.coach.List
			if(isLastPath) {
				for(var i=0; i<dataObject.length(); i++) {
					var itemPlainValue = dataBindingHelper.getPlainValue(dataObject, i, true);
					if(compareFunc.call(thisObject, itemPlainValue)) {
						return true;
					}
				}
			} else {
				var newIndex = index + 1;
				for(var j=0; j<dataObject.length(); j++) {
					// newDataObject is a Stateful object or List object
					var newDataObject = dataObject.get(j);
					if(coachUtil._traverse(pathArray, newIndex, newDataObject, compareFunc, thisObject)) {
						return true;
					}
				}
			}
			return false;
		} else {
			// dataObject represents a Stateful object usually. Could be List object when path is "listSelected"
			if(isLastPath) {
				var itemPlainValue = dataBindingHelper.getPlainValue(dataObject, path);
				return compareFunc.call(thisObject, itemPlainValue);
			} else {
				// newDataObject is a Stateful object or List object
				var newDataObject = dataObject.get(path);
				return coachUtil._traverse(pathArray, index+1, newDataObject, compareFunc, thisObject);
			}
		}
	}  	
  	
	return coachUtil;
});
