/*
Licensed Materials - Property of IBM
5725-C95
 Copyright IBM Corporation 2012. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

// utilities around CoachView

define(["dojo/dom-class", "./CONSTANTS", "./domHelper"], function(domClass, CONSTANTS, domHelper) {
				
	var coachViewHelper = new Object();

	coachViewHelper.getViewIdMetaData = function(viewScope) {
		if(!!viewScope) {
			var metaData = new Object();
			var botomUpViewLevels = new Array();
			var currentView = viewScope;
			while(!!currentView) {
				var viewId = currentView.context.viewid;
				var parentView = currentView.context.parentView();
				if(!parentView) {
					// currentView is already topLevelView
					metaData[CONSTANTS.TOPLEVEL_VIEW_ID] = viewId;
				} else {
					var oneLevel = new Object();
					oneLevel[CONSTANTS.SUBVIEW_ID] = viewId;
					var siblings = parentView.context.getSubview(viewId, true);
					for(var k=0; k<siblings.length; k++) {
						if(siblings[k] === currentView) {
							oneLevel[CONSTANTS.SUBVIEW_INDEX] = k;
							break;
						}
					}
					botomUpViewLevels.push(oneLevel);					
				}
				currentView = parentView;
			}
			if(botomUpViewLevels.length > 0) {
				metaData[CONSTANTS.VIEW_PATH] = botomUpViewLevels.reverse();
			}
			
			return metaData;
		}
		
		return null;
	}
	
	coachViewHelper.hasSameStructure = function(viewIdMetaData1, viewIdMetaData2) {
		if(!!viewIdMetaData1 && !!viewIdMetaData2) {
			if(viewIdMetaData1[CONSTANTS.TOPLEVEL_VIEW_ID] === viewIdMetaData2[CONSTANTS.TOPLEVEL_VIEW_ID]
				&& !!viewIdMetaData1[CONSTANTS.VIEW_PATH] && !!viewIdMetaData2[CONSTANTS.VIEW_PATH]
				&& viewIdMetaData1[CONSTANTS.VIEW_PATH].length == viewIdMetaData2[CONSTANTS.VIEW_PATH].length) {
				
				for(var k=0; k<viewIdMetaData1[CONSTANTS.VIEW_PATH].length; k++) {
					if(viewIdMetaData1[CONSTANTS.VIEW_PATH][k][CONSTANTS.SUBVIEW_ID] != viewIdMetaData2[CONSTANTS.VIEW_PATH][k][CONSTANTS.SUBVIEW_ID]) {
						return false;
					}
				}
				// now we know the structures are same, although CONSTANTS.SUBVIEW_INDEX might be different 
				return true;
			}			
		}
		
		return false;
	}
	
	
	coachViewHelper.findViewByLocation = function(viewIdMetaData, topLevelViewScope) {
		var topLevelViewId = viewIdMetaData[CONSTANTS.TOPLEVEL_VIEW_ID];
		var topLevelView = topLevelViewScope[topLevelViewId];
		var viewPath = viewIdMetaData[CONSTANTS.VIEW_PATH];
		if(!!viewPath) {
			var currentView = topLevelView;
			for(var k=0; k<viewPath.length; k++) {
				var oneLevel = viewPath[k];
				var subviews = currentView.context.getSubview(oneLevel[CONSTANTS.SUBVIEW_ID], true);
				var index = oneLevel[CONSTANTS.SUBVIEW_INDEX];
				if(subviews.length <= index) {
					// something wrong. no way to continue
					return null;
				} else {
					currentView = subviews[index];
				}
			}
			return currentView;
		} else {
			// topLevelView is what we are looking for
			return topLevelView;
		}
	}
	
	
	coachViewHelper.findViewScope = function(domId, topLevelViewScope) {

		var domNode = document.getElementById(domId);
		
		var nodeChain = new Array();
		var classAttribute;
		while(!!domNode && domNode.tagName != "BODY") {
			if(domClass.contains(domNode, CONSTANTS.TOP_LEVEL)) {
				// now domNode is the topLevel view
				var topLevelViewId = domNode.getAttribute(CONSTANTS.VIEW_ID);
				var topViewScope = topLevelViewScope[topLevelViewId];			
				return coachViewHelper._findViewScopeByChain(topViewScope, nodeChain);
			}
			nodeChain[nodeChain.length] = domNode;
			domNode = domNode.parentNode;
		}
		
		return null;	
	}

	// find the direct parent view for the viewDivNode
	coachViewHelper.findParentViewScope = function(rootViewScope, viewDivNode) {
		// we intend not to have robust checking on the while loops in ths method. If exception occurs, we will find some scenarios that we haven't thought of. 
		
		var parentViewScopeNode = null;
		var rootDivNode = rootViewScope.context.element;
		var parentNode = viewDivNode.parentNode;
		while(!parentViewScopeNode) {
			if(parentNode == rootDivNode) {
				// well, rootViewScope is the direct owner
				return rootViewScope;
			}
			if(domClass.contains(parentNode, CONSTANTS.COACH_VIEW)) {
				parentViewScopeNode = parentNode;
				break;
			}
			parentNode = parentNode.parentNode;
		}
		
		// try to find the ViewScope object of parentViewScopeNode with the help of rootViewScope		
		var nodeChain = new Array();
		nodeChain[0] = parentViewScopeNode;
		parentNode = parentViewScopeNode.parentNode;
		while(parentNode != rootDivNode) {
			nodeChain[nodeChain.length] = parentNode;
			parentNode = parentNode.parentNode;			
		}
		
		return coachViewHelper._findViewScopeByChain(rootViewScope, nodeChain);
	}

	coachViewHelper._findViewScopeByChain = function(rootViewScope, nodeChain) {
		// we intend not to have robust checking on the for loop in ths method. If exception occurs, we will find some scenarios that we haven't thought of. 

		if(!rootViewScope || !nodeChain) {
			return null;
		}
		
		var tempViewScope = rootViewScope;
		for(var k=nodeChain.length -1; k>=0; k--) {
			var viewId = nodeChain[k].getAttribute(CONSTANTS.VIEW_ID);
			var divId = nodeChain[k].getAttribute("id");
			if(!!viewId && !!divId) {
				if(!tempViewScope.context.subview[viewId]) {
					// console.debug("*****ERROR: Developer, viewId [" + viewId + "] may not the right one");
					return null;
				} else if(!tempViewScope.context.subview[viewId][divId]) {
					// console.debug("*****ERROR: Developer, divId [" + divId + "] may not the right one");
					return null;
				} 
				tempViewScope = tempViewScope.context.subview[viewId][divId];
				if(!tempViewScope) {
					return null;
				}
			}
		}
		return tempViewScope;
	}
	
	coachViewHelper.findAllViewInstances = function(topLevelViews, viewInstancePath) {
		var viewInstances = [];
		if(!!viewInstancePath && viewInstancePath.length > 0) {
			var topLevelView = topLevelViews[viewInstancePath[0]];
			coachViewHelper._collectViewInstances(viewInstances, topLevelView, viewInstancePath, 1);
		}
		return viewInstances;
	}
	
	coachViewHelper._collectViewInstances = function(viewInstances, currentViewInstance, viewInstancePath, index) {
		if(!currentViewInstance) {
			return;
		}
		
		if(index >= viewInstancePath.length) {
			// finish all path
			viewInstances.push(currentViewInstance);
		} else {
			var viewsWithSameViewId = currentViewInstance.context.getSubview(viewInstancePath[index]);
			var newIndex = index + 1;
			for(var k=0; k<viewsWithSameViewId.length; k++) {
				coachViewHelper._collectViewInstances(viewInstances, viewsWithSameViewId[k], viewInstancePath, newIndex);
			}
		}
	}
	
	/*	
		This function returns the coach view instance that is described by the input parameter "viewInstancePath" which is a list of control ids.   
		The list of control ids should completely specify the view hierarchy, starting from the top level view, 
		all the way down to the coach view whose instance is expected to be returned.
		
		If  the list of control id doesn't describe the entire view hierarchy properly, this function returns null.			
		If a coach view repeats, first instance of that view is chose to continue.
	*/
	coachViewHelper.findFirstViewInstance = function(topLevelViews, viewInstancePath) {
		try {
			if(!!topLevelViews && !!viewInstancePath && viewInstancePath.length > 0) {
				var topLevelView = topLevelViews[viewInstancePath[0]];
				return coachViewHelper._findFirstViewInstance(topLevelView, viewInstancePath, 1);
			}
		} catch(ex) {
			// need to do nothing
		}
		
		return null;
	}
	
	coachViewHelper._findFirstViewInstance = function(currentViewInstance, viewInstancePath, index) {
		if(!currentViewInstance) {
			return null;
		}
		
		if(index >= viewInstancePath.length) {
			// finish all path
			return currentViewInstance;
		} else {
			var viewsWithSameViewId = currentViewInstance.context.getSubview(viewInstancePath[index]);
			if(viewsWithSameViewId.length > 0) {
				var newIndex = index + 1;
				return coachViewHelper._findFirstViewInstance(viewsWithSameViewId[0], viewInstancePath, newIndex);
			} else {
				// no subview
				return null;
			}
		}
	}
	
	coachViewHelper.getViewPath = function(viewInstance) {
		var path = [];
		var aView = viewInstance;
		while(!!aView) {
			path.push(aView.context.viewid);
			aView = aView.context.parentView();
		}
		return path.reverse();
	}
	
	// Find the subviews of specified ownerView, those subviews should reside in specified contentBox (contentBoxDomNode) 
	// directly or indirectly (e.g. subview is in another contentBox which is in specified contentBox)
	// As the return, subviews are put in the provided subviewArray
	coachViewHelper.findSubviews = function(subviewArray, ownerView, contentBoxDomNode) {
		// handle nested Views
		var immediateViewList = domHelper.shallowQueryByClass(contentBoxDomNode, CONSTANTS.COACH_VIEW, false);
		var viewId, domId;
		for(var i=0; i<immediateViewList.length; i++) {
			var domId = immediateViewList[i].getAttribute("id");
			var viewId = immediateViewList[i].getAttribute(CONSTANTS.VIEW_ID);
			if(ownerView.context.subview[viewId] && ownerView.context.subview[viewId][domId]) {
				subviewArray.push(ownerView.context.subview[viewId][domId]);
			}
		}
		
		// handle nested ContentBox
		var immediateContentBoxList = domHelper.shallowQueryByClass(contentBoxDomNode, CONSTANTS.CONTENT_BOX, false);
		for(var i=0; i<immediateContentBoxList.length; i++) {
			coachViewHelper.findSubviews(subviewArray, ownerView, immediateContentBoxList[i]);
		}			
	}	
	
	return coachViewHelper;
});
