/*
Licensed Materials - Property of IBM
5725-C95
 Copyright IBM Corporation 2012. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

// manage the popup for collaboration

define(["dojo/query", "dijit/popup", "dijit/TooltipDialog", "dojo/dom-geometry", "dojo/window", "dojo/has", "dojo/_base/sniff"], function(query, popup, TooltipDialog, domGeometry, dojoWindow, has) {

	var popupManager = new Object();
	
	popupManager.CONSTANTS = {
		// css class names
		COLLABORATOR_INFO:	"CoachView_collaboratorInfo",
		COLLABORATOR:		"CoachView_collaborator",
		DATA_CHANGED:		"CoachView_dataChanged"
	};
	
	function scrollIntoView(node, viewNode) {
	
		_scrollIntoView(node);
		
		if(domGeometry.position(node).y == 0 && viewNode.id != node.id) {			
			// if node's y position is still 0 after we try to scrollIntoView, node could be invisible.
			// For those cases, we try on the node of coachView instead
			_scrollIntoView(viewNode);
		}	     		
	}
	
	function _scrollIntoView(node) {
/*	
	     var vs = dojoWindow.getBox();
	     console.log("node.id is " + node.id);
	     console.log("vs.h is " + vs.h);
	     console.log("before doing anything, domGeometry.position(node) is " + domGeometry.position(node).y);
*/
		if(has("ie")) {
			node.scrollIntoView(false);
	     	// console.log("after node.scrollIntoView(false), domGeometry.position(node) is " + domGeometry.position(node).y);
			
			// HTML's scrollIntoView may cut the left edge of node on ie
			// dojo's scrollIntoView has problem to handle the scrollbar (horizontal scrollbar covers part of node after scrollIntoView)
			// As the result, we have to call both for ie
			dojoWindow.scrollIntoView(node);	
		    // console.log("after dojoWindow.scrollIntoView(node), domGeometry.position(node) is " + domGeometry.position(node).y);
		} else {
			node.scrollIntoView(false);
	     	// console.log("after node.scrollIntoView(false), domGeometry.position(node) is " + domGeometry.position(node).y);
		}
	}
		
	popupManager.showCollaboratorInfo = function(node, color, bgColor, message, viewNode) {
	    // make sure node is visible in current viewport
		scrollIntoView(node, viewNode);
		
		var newContent = "<div class='" + popupManager.CONSTANTS.COLLABORATOR_INFO + "' style=" + "'color: " + color + ";background-color: " + bgColor + ";'><label>"  + message + "</label></div>";

	    var tooltipDialog = new TooltipDialog({
	        content: newContent 
	    });
	    
	    // have to open and close the TooltipDialog to let it fully populated then we re-style it
		popup.open({ 
	        popup: tooltipDialog, 
	        around: node, 
	        orient: ["above-alt", "above", "below-alt", "below"]
	        // orient: {TR: 'BR', BL: 'BR'} // this orient works on dojo 1.6                           
	    }); 	    
	    popup.close(tooltipDialog);
	    
	    var customClass = popupManager.CONSTANTS.COLLABORATOR;
	    var domNodeOfPopup = findRootDivOfPopup(tooltipDialog);
	    if (!!domNodeOfPopup) {
	    	    var queryNode = query('.dijitTooltipContainer', domNodeOfPopup);
	    	    if (!!queryNode && !!(queryNode.addClass)) queryNode.addClass(customClass);
	    	    queryNode = query('.dijitTooltipFocusNode', domNodeOfPopup); 
	    	    if (!!queryNode && !!(queryNode.addClass)) queryNode.addClass(customClass);
	    	    queryNode = query('.dijitTooltipAbove', domNodeOfPopup); 
	    	    if (!!queryNode && !!(queryNode.addClass)) queryNode.addClass(customClass);
	    	    queryNode = query('.dijitTooltipBelow', domNodeOfPopup); 
	    	    if (!!queryNode && !!(queryNode.addClass)) queryNode.addClass(customClass);
	    	    queryNode = query('.dijitTooltipConnector', domNodeOfPopup);
	    	    if (!!queryNode && !!(queryNode.addClass)) queryNode.addClass(customClass);
	    }
	
		popup.open({ 
	        popup: tooltipDialog, 
	        around: node, 
	        orient: ["above-alt", "above", "below-alt", "below"]
	    }); 

		require(["dojo/_base/fx"], function(baseFx){				
			  baseFx.fadeOut({
				   node: domNodeOfPopup,
				   duration: 5000,
				   onEnd: function() {
				       popup.close(tooltipDialog);
				       domNodeOfPopup.parentNode.removeChild(domNodeOfPopup);
				       domNodeOfPopup = null;
				   }
			   }).play();
		});
	}
	
	popupManager.hideCollaboratorInfo = function(node) {
		// popup goes away automatically. do need to do nothing at this moment
	}
	
	popupManager.showDataChange = function(node, message, viewNode) {

	    // make sure node is visible in current viewport
    	scrollIntoView(node, viewNode)

		var newContent = "<label>" + message + "</label>";				

	    var tooltipDialog = new TooltipDialog({
	        content: newContent 
	    });
	    
		popup.open({ 
	        popup: tooltipDialog, 
	        around: node, 
	        orient: ["above-alt", "above", "below-alt", "below"]
	    }); 
	    
	    var customClass = popupManager.CONSTANTS.DATA_CHANGED;
	    var domNodeOfPopup = findRootDivOfPopup(tooltipDialog);
	    if (!!domNodeOfPopup) {
		var queryNode = query('.dijitTooltipContainer', domNodeOfPopup);
		if (!!queryNode && !!(queryNode.addClass)) queryNode.addClass(customClass);
		var queryNode = query('.dijitTooltipConnector', domNodeOfPopup);
		if (!!queryNode && !!(queryNode.addClass)) queryNode.addClass(customClass);
	    }
			    
		require(["dojo/_base/fx", "dojo/fx"], function(baseFx, fx){			
			var outlineColor = !!node.style.outline ? node.style.outlineColor : null;
			var fadeOutPopup = baseFx.fadeOut({
				   node: domNodeOfPopup,
				   duration: 1000,
				   onEnd: function() {
				       popup.close(tooltipDialog);
				       domNodeOfPopup.parentNode.removeChild(domNodeOfPopup);
				       domNodeOfPopup = null;
				       if(!!node.style.outline) {
				           node.style.outlineColor = outlineColor;
				       }
				   }
				 });
			var fadeOutOutline = baseFx.animateProperty({
				   node:node,
				   duration: 1000,
				   properties: {
				     outlineColor: {end: "null"},
				   }
				});
			var fadeInOutline = baseFx.animateProperty({
				   node:node,
				   duration: 1000,
				   properties: {
				     outlineColor: {end: outlineColor},
				   }
				});
					 
			if(!!node.style.outline) {
				fx.chain([fadeOutOutline, fadeInOutline, fadeOutOutline, fadeOutPopup]).play();			
			} else {
				fadeOutPopup.play();
			}			
		});
	}	

	function findRootDivOfPopup(tooltipDialog) {
		// update accordingly if the DOM node of tooltipDialog changes in the future dojo release
		return tooltipDialog.domNode.parentNode;
	/*
		var bubbleDiv;	
	    var dijitTooltipDialogPopups = query('.dijitTooltipDialogPopup');
	    if(dijitTooltipDialogPopups.length == 1) {
	    	bubbleDiv = dijitTooltipDialogPopups[0];
	    } else {
	    	for(var k=0; k<dijitTooltipDialogPopups.length; k++) {
	    		if(query('.bubbleNode', dijitTooltipDialogPopups[k]).length > 0) {
	    			bubbleDiv = dijitTooltipDialogPopups[k];
	    			break;
	    		};
	    	};
	    }	    
	    return bubbleDiv;
	*/
	}                             

		
	return popupManager;

});
