/*
Licensed Materials - Property of IBM
5725-C95
 Copyright IBM Corporation 2012. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

// Contains all common utilities

define(["dojo/_base/kernel","./CONSTANTS"], function(dojoKernel, CONSTANTS) {
				
	var commonHelper = new Object();
	
	commonHelper.INFO = "info";
	commonHelper.WARN = "warn";
	commonHelper.ERROR = "error";
	commonHelper.LOG = "log";
	
	// this API write the message directly to browser's console as long as browser's console exists
	commonHelper.consoleLog = function(message, type) {
		if(!console) {
			return;
		}
		
		if(commonHelper.INFO === type && !!console.info) {
			console.info(message);
		} else if(commonHelper.WARN === type && !!console.warn) {
			console.warn(message);
		} else if(commonHelper.ERROR === type && !!console.error) {
			console.error(message);
		} else if(!!console.log){
			console.log(message);
		}
	}
	
	commonHelper.timestamp = function(action) {
		if(commonHelper.isDebugMode()) {
			commonHelper.consoleLog("Performance: " + (new Date()).getTime() + " -- " + action, commonHelper.INFO);
		}
	}
	
	commonHelper.isDebugMode = function() {
		if(!!dojoKernel.config) {
			return dojoKernel.config.isDebug;
		}
		
		return false;
	}
	
	commonHelper.getDojoVersion = function() {
		return dojoKernel.version;
	}	
	
	commonHelper.shallowCloneMap = function(aMap) {
		var clonedMap = {};
		for(var key in aMap) {
			clonedMap[key] = aMap[key];
		}
		return clonedMap;
	}
	
	commonHelper.isMapEmpty = function(map) {
	   for(var key in map) {
		  if (map.hasOwnProperty(key)) {
			 return false;
		  }
	   }
	   return true;
	}
	
	// return value of commonHelper.isValuesEqual()
	commonHelper.YES = 1;
	commonHelper.UNKNOWN = 0;
	commonHelper.NO = -1;
	
	/*
	 Compare two provided values.
	 return -
	 	commonHelper.YES: value1 is equal to value2
	 	commonHelper.NO: value1 is not equal to value2.
	 	commonHelper.UNKNOWN: the brief check can not determine if value1 and value2 are equal
	*/
	commonHelper.isValuesEqual = function(value1, value2) {
		// only compare when value1 and value2 are primitive types.
		// if value1 and value2 points to same object, this function returns commonHelper.UNKNOWN instead of commonHelper.YES 
		// in the sake of backward compatibility (some stock control didn't use coach API properly in release 8000 and 8010)
		var typeOfValue = typeof(value1);
		if(typeOfValue == 'string' || typeOfValue == 'number' || typeOfValue == 'boolean' || typeOfValue == 'undefined' || typeOfValue == 'null') {
			if(value1 === value2) {
				return commonHelper.YES;
			} else {
				return commonHelper.NO;
			}
		}
		return commonHelper.UNKNOWN;
	}
	
	/*
	 Check if bindingStrA starts with bindingStrB in term of path.
	 If the data represented by bindingStrA is sub-object of the data represented by bindingStrB, return true. 
	 If bindingStrA is equal to bindingStrB, return true if includeEqualCase is true, Otherwise return false.
	 Return false for all other cases.
	 
	 Example:
	 	bindingStrA			bindingStrB		return
	 	-----------------------------------------------
	 	local.abc			local.abc		true(if includeEqualCase == true), false otherwise
	 	*					null/undefined	false;
	 	null/undefined		*				false;
	 	local.def			local.abc		false
	 	local.abcdef		local.abc		false
	 	local.abc.def		local.abc		true
	 	local.abc.d.e		local.abc		true
	 	local.abc[2]		local.abc		true
	*/
	commonHelper.startsWithPath = function(bindingStrA, bindingStrB, includeEqualCase) {
		if(bindingStrA == bindingStrB) {
			if(includeEqualCase) {
				return true;
			}
		} else if(!!bindingStrA && !!bindingStrB && (bindingStrA.indexOf(bindingStrB) == 0)) {
			// still need fine check, because ("local.abcdef", "local.abc") also passes above test
			var difference = bindingStrA.replace(bindingStrB, "");
			// difference is not blank string because bindingStrA != bindingStrB
			var firstCharacter = difference.substring(0, 1);
			
			if(firstCharacter == "." || firstCharacter == "[") {
				//("local.abc.def", "local.abc"), or ("local.abc[2]", "local.abc")
				return true;
			}
		}
		
		return false;
	}		
	
	commonHelper.timestamp("commonHelper is created");
	
	return commonHelper;
});
