/*
Licensed Materials - Property of IBM
5725-C95
(C) Copyright IBM Corporation 2012, 2014. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

// Contains all utilities that Coach runtime supports validation

define(["require", "dojo/_base/lang", "./CONSTANTS", "./bindingResolver"], function(require, lang, CONSTANTS, bindingResolver) {
				
	var dataBindingHelper = new Object();
	
	// rtDataBinding provides 3 helper method for resolving List related data-binding during runtime.
	dataBindingHelper.rtDataBinding = new Object();
	dataBindingHelper.rtDataBinding._get = function(viewElement) {
		var rtBinding = viewElement.getAttribute(CONSTANTS.DATA_BINDING_RT);
		if(!rtBinding) {
			return viewElement.getAttribute(CONSTANTS.DATA_BINDING);
		} else {
			return rtBinding;
		}
	}	
	dataBindingHelper.rtDataBinding._set = function(viewElement, bindingValue) {
		viewElement.setAttribute(CONSTANTS.DATA_BINDING_RT, bindingValue);
	}
	dataBindingHelper.rtDataBinding._delete = function(viewElement) {	
		// viewElement.removeAttribute(CONSTANTS.DATA_BINDING_RT);
	}
	

	// original regular expression which does not include the ^(start of string), the $ (end of string), or milliseconds
	// [0-9]{4}(\-[0-9]{2}){2}T([0-9]{2}\:){2}[0-9]{2}
	
	// updated original regex but it is still missing the timezone portion, has optional milliseconds,
	// and allows for invalid months, days, hours, minutes, seconds (i.e. 99 months is invalid according to ECMAScript)  
	// ^[0-9]{4}(\-[0-9]{2}){2}T([0-9]{2}\:){2}[0-9]{2}(\.[0-9]{3})?$

	// ISO 8601 (W3 subset), this is too forgive for us to use since 2014 or 2014-02 as examples pass through 
	// ^(\d{4}((-)?(0[1-9]|1[0-2])((-)?(0[1-9]|[1-2][0-9]|3[0-1])(T(24:00(:00(\.[0]+)?)?|(([0-1][0-9]|2[0-3])(:)[0-5][0-9])((:)[0-5][0-9](\.[\d]+)?)?)((\+|-)(14:00|(0[0-9]|1[0-3])(:)[0-5][0-9])|Z))?)?)?)$

	// ISO 8601 (W3 subset) but forces an almost complete string with valid segments, the milliseconds and timezone portion are optional
	// ^\d{4}-(0[1-9]|1[0-2])-(0[1-9]|[1-2][0-9]|3[0-1])T((24:00:00\.000)|(([0-1][0-9]|2[0-3])(:)[0-5][0-9](:)[0-5][0-9](\.\d{3})?))(((\+|-)(14:00|(0[0-9]|1[0-3])(:)[0-5][0-9]))|Z)?$

	var testDate = new Date("2014-04-25T12:12:12.123Z");
	dataBindingHelper._useNativeDateCtor = testDate.toString() == "NaN" || testDate.toString() == "Invalid Date" ? false : true;

	dataBindingHelper._isoDateTimeRegExp = /^\d{4}-(0[1-9]|1[0-2])-(0[1-9]|[1-2][0-9]|3[0-1])T((24:00:00\.000)|(([0-1][0-9]|2[0-3])(:)[0-5][0-9](:)[0-5][0-9](\.\d{3})?))(((\+|-)(14:00|(0[0-9]|1[0-3])(:)[0-5][0-9]))|Z)?$/;

	dataBindingHelper.isDateString = function(str) {
		var result = false;
		if (dataBindingHelper._isoDateTimeRegExp.test(str)) {
			result = true;
		}
		return result;
	}

	dataBindingHelper.stringToDate = function (dateStr) {
		// the date string will be of the one of the following forms:
		// yyyy-mm-ddThh:mm:ss.sss
		// yyyy-mm-ddThh:mm:ss.sssZ
		// yyyy-mm-ddThh:mm:ss.sss-05:00
		// yyyy-mm-ddThh:mm:ss.sss+05:30
		// yyyy-mm-ddThh:mm:ss
		// yyyy-mm-ddThh:mm:ssZ
		// yyyy-mm-ddThh:mm:ss-05:00
		// yyyy-mm-ddThh:mm:ss+05:30

		if (dataBindingHelper._useNativeDateCtor) {
			// Only works on IE9+, FF, Chrome
			var result = new Date(dateStr);
			return result;
		}

		// This code path is only for browsers that do not natively support the ISO 8601 format, like IE8

		var datetime = dateStr.split("T");
		var date = datetime[0].split("-");
				
		var hasMS = datetime[1].indexOf(".") != -1;
		
		var timeSegs = [];
		if (hasMS) {
			timeSegs = datetime[1].split(".");
		} else {
			timeSegs.push(datetime[1].slice(0,8));
			timeSegs.push(datetime[1].slice(8));
		}
				
		var time = timeSegs[0].split(":");

		var offset = 0; // how many ms to offset epochValue by
		var idx = timeSegs[1].indexOf("Z");
		var factor = 1; // factor to apply to offset so it is an add or subtract
		var utc = true; // whether we are dealing is "utc" date (aka Zulu)
		if (idx == -1) {
			utc = false;
			factor = 1;
			idx = timeSegs[1].indexOf("-");
		}
		if (idx == -1) {
			utc = false;
			factor = -1;
			idx = timeSegs[1].indexOf("+");
		}
		if (hasMS) {
			if (idx != -1) {
				time.push(timeSegs[1].slice(0,idx));
			} else {
				// no Z, -, or + found so we only have the milliseconds
				time.push(timeSegs[1]);
			}
		} else {
			// there are no ms in the dateStr
			time.push("000");
		}
		if (idx != -1 && !utc) {
			var tzInfo = timeSegs[1].slice(idx+1);
			var tz = tzInfo.split(":")
			offset = parseInt(tz[0])*60 + parseInt(tz[1]);
			offset = (offset * 60 * 1000) * factor;
		}

		var epochValue = Date.UTC(parseInt(date[0]), parseInt(date[1],10)-1, parseInt(date[2],10),
			parseInt(time[0],10), parseInt(time[1],10), parseInt(time[2],10), parseInt(time[3],10));
		var result = new Date(epochValue + offset);

		return result;
	}

	dataBindingHelper.collectBpmIds = function(bindingStr) {
		var bpmIds = null;
		if(bindingStr.indexOf("[") == -1) {
			return bpmIds;
		}
		var pathArray = bindingStr.split(".");
		var tempBindingStr = null;
		for (var i=0; i<pathArray.length; i++) {
			var aPath = pathArray[i];
			var leftIndexPosition = aPath.indexOf("[");
			if (leftIndexPosition != -1) {
				var rightIndexPosition = aPath.indexOf("]");
				if(rightIndexPosition - leftIndexPosition > 1) {
					// encounters abc[n], need to find out bpm_id from the index (n)
					var indexValue = aPath.substring(leftIndexPosition+1, rightIndexPosition);
					var listShortName = aPath.substring(0, leftIndexPosition);
					var listBindingStr = !tempBindingStr ? listShortName : tempBindingStr + "." + listShortName;
					var resolvedBinding = bindingResolver.resolveBinding(listBindingStr);
					if(!resolvedBinding) {
						// can't resolve this binding. The rest of bindings won't be resolved for sure. So stop here.
						break;
					}
					var list = resolvedBinding.get("value");
					var bpmId = list.indexToId[indexValue];
					if(!bpmIds) {
						bpmIds = new Array();
					}
					bpmIds.push(bpmId);
				}
			}
			
			if(!tempBindingStr) {
				tempBindingStr = aPath;
			} else {
				tempBindingStr += "." + aPath;
			}
		}
		
		return bpmIds;
	}

	// update RTDataBinding for specified view and its all descendants views.
	dataBindingHelper.updateAllRtDataBinding = function(viewScope, bindingStrToListMap) {
    	if(!!viewScope._bpmIds) {
    		if(!bindingStrToListMap) {
    			bindingStrToListMap = {};
    		}
			var failToResolveIndex = false;
    		var updatedBindingStr = viewScope.context.element.getAttribute(CONSTANTS.DATA_BINDING);
			var indexWithBracket = "[" + CONSTANTS.CURRENT_ITEM + "]";
    		for(var k=0; k<viewScope._bpmIds.length; k++) {
    			var position = updatedBindingStr.indexOf(indexWithBracket);
    			var listBindingStr = updatedBindingStr.substring(0, position);
				var list = bindingStrToListMap[listBindingStr];
				if(!list) {
					list = bindingResolver.resolveBinding(listBindingStr).get("value");
					bindingStrToListMap[listBindingStr] = list;
				}
				var currentIndex = list.idToIndex(viewScope._bpmIds[k]);
				if(currentIndex == null || typeof currentIndex == "undefined") {
					// example scenario: list item is deleted, but table row has not been removed. table cell (coach view) still remembers the bpmId that has no matching index in the list.
					failToResolveIndex = true;
					break;
				}
    			var replaceValue = "[" + currentIndex + "]";
    			updatedBindingStr = updatedBindingStr.replace(indexWithBracket, replaceValue);
    		}
    		
			if(!failToResolveIndex) {
				// let view instance know the new bindingStr
				viewScope._setRtDataBindingStr(updatedBindingStr);
			}
    	}
		
		if (!!viewScope.context.subview) {
			for (var viewId in viewScope.context.subview) {
				var subviews = viewScope.context.subview[viewId];
				for(var divId in subviews) {
					dataBindingHelper.updateAllRtDataBinding(subviews[divId], bindingStrToListMap);
				}
			}		
		}			
	}
	
	// this function is NOT called at all so far
	// make sure every RTDataBinding in current coach is up to date
	dataBindingHelper.updateRtDataBindingGlobally = function() {
		var bindingStrToListMap = new Object();
		var topLevelViews = bindingResolver.globalAssets.topLevelViewScope;
		for (var viewId in topLevelViews) {
			var viewScope = topLevelViews[viewId];
			dataBindingHelper.updateAllRtDataBinding(viewScope, bindingStrToListMap);
		}
	}

	// it's array if end with "[]", otherwise resolve the binding to check if it is type of com.ibm.bpm.coach.List
	dataBindingHelper.isBindingArray = function(bindingStr) {
		var refArray = bindingStr.split(".");
		if(refArray[0] == "local") {
			return (bindingStr.substring(bindingStr.length - 2) == "[]");
		} else {
			// could be config option
			var binding = bindingResolver.resolveBinding(bindingStr);
			if(!!binding && binding.get("value") instanceof com.ibm.bpm.coach.List) {
				return true;
			}
		}
			
		return false;
	}	
	
	dataBindingHelper.isNumber = function(stringValue) {
		var convertToNumber = Number(stringValue);
		return !isNaN(convertToNumber);
	}
	
	dataBindingHelper.getNumberOfIndex = function(bindingStr) {
		var pathArray = bindingStr.split(".");
		var indexWithBracket = "[" + CONSTANTS.CURRENT_ITEM + "]";
		var numOfIndex = 0;
		for(var k=0; k<pathArray.length; k++) {
			if(pathArray[k].indexOf(indexWithBracket) != -1) {
				numOfIndex++;
			}			
		}		
		return numOfIndex;
	}
	
	dataBindingHelper.updateBindingStr = function(bindingStr, bindingIndex, maxNumOfUnresolvedRepeating, numOfIndex) {
		if(!numOfIndex) {
			numOfIndex = dataBindingHelper.getNumberOfIndex(bindingStr);
		}
		
		if(numOfIndex >= maxNumOfUnresolvedRepeating) {
			var indexWithBracket = "[" + CONSTANTS.CURRENT_ITEM + "]";
			var replaceValue = "[" + bindingIndex + "]";
			var newBinding;
			if(maxNumOfUnresolvedRepeating == 1 && numOfIndex > 1) {
				// assign all the outstanding "[index]" with current bindingIndex since this is the last repeating index
				newBinding = bindingStr;
				for(var k=0; k<numOfIndex; k++) {
					newBinding =newBinding.replace(indexWithBracket, replaceValue);
				}
			} else {
				newBinding = bindingStr.replace(indexWithBracket, replaceValue); // replace very first "[index]"
			}
			
			return newBinding; 
		} else {
			// now numOfIndex < maxNumOfUnresolvedRepeating. the unresolved "[index]" should be reserved for the following repeating
			return bindingStr;
		}				
	
	}
	
	// return the plain value (could be complex object) for the specified property on the Stateful or List object
	dataBindingHelper.getPlainValue = function(statefulOrListObject, property, isIndex) {
		if(!!statefulOrListObject) {
			if (statefulOrListObject instanceof com.ibm.bpm.coach.List) {
				// List
				if(parseInt(property) >= 0) {
					// List item
                    if(isIndex) {
                        return statefulOrListObject.items[property];
                    } else {
					    var index = statefulOrListObject.idToIndex(parseInt(property));
					    return statefulOrListObject.items[index];
					}
				} else {
					// List property, e.g listSelected
					return statefulOrListObject[property];
				}
			} else {
				// Stateful
				return statefulOrListObject[property];
			}
		}
		return null;
	}
	
	// Example, bindingPath is "local.a.b.c", name is "d", value is "value of new d".
	// set("d", "value of new d") just occurred on the Stateful object represented by "local.a.b.c". So the plain value of d on this Stateful object is up to date.
	// However, the Stateful object representd by "local.a.b" has plain value of c which contains plain value of d. 
	// Dojo Stateful doesn't update the plain value c.d under "local.a.b", b.c.d under "local.a", and a.b.c.d under "local". This function updates those plain values.
	dataBindingHelper.updatePlainValueInStatefulChain = function(bindingPath, name, value) {
		var pathArray = dataBindingHelper._computePaths(bindingPath);
		// Example of pathArray: 
		// "local.a.b"							-> ["local", "a", "b"]
		// "local.a.b[1]"						-> ["local", "a", "b", "[1]"]
		// "local.a.b[1].c[2].d"				-> ["local", "a", "b", "[1]", "c", "[2]", "d"]
		// "config.config1.myOption"			-> ["config", "config1", "myOption"]
		// "config.config2._metadata.label"		-> ["config", "config2", "_metadata", "label"]
		// "config.config3.myOption.p1"			-> ["config", "config3", "myOption", "p1"]
		// "config.config3.myListOption[2].p1"	-> ["config", "config3", "myListOption", "[2]", "p1"]
		
		var statefulObject;
		// For pathArray ["local", "a", "b"], b represents a Stateful object. the plain value of "name" (e.g. "c")  on that Stateful object is already up to date.
		// So, last Stateful object that needs to update plain value "c" is "b" plain object stored in Stateful represented by a.
		var lastPathIndex = pathArray.length - 1;
		var firstPathIndex;
		
		if(pathArray[0] == "local") {
			firstPathIndex = 1;
			statefulObject = bindingResolver.globalAssets["local"];
		} else if(pathArray[0] == "config"){
			// first 2 items in pathArray don't represent Stateful object for non-metadata option, e.g. ["config", "config1", "myOption"]
			// first 3 items in pathArray don't represent Stateful object for metadata option, e.g. ["config", "config2", "_metadata", "label"]
			var pathForRootStateful = pathArray[0];
			if(pathArray[2] == "_metadata") {
				firstPathIndex = 4;
			} else {
				firstPathIndex = 3;
			}
			
			if(firstPathIndex >= pathArray.length) {
				// nothing needs to update
				statefulObject = null;
			} else {			
				for(var k=1; k<firstPathIndex; k++) {
					pathForRootStateful += "." + pathArray[k];
				}
				// RTC PMR 219276 - PMR: 40425,057,649 
				// statefulObject = bindingResolver.resolveBinding(pathForRootStateful);
				// bindingResolver.resolveBinding() returns wrapper object, not statefulObject
				var wrapperObject = bindingResolver.resolveBinding(pathForRootStateful);
				if(wrapperObject) {
				    statefulObject = wrapperObject.boundObject;
				}
			}
		}
		
		if(statefulObject && firstPathIndex <= lastPathIndex) {
			var clonedPlainValue = lang.clone(value);
			for(var k=firstPathIndex; k<=lastPathIndex; k++) {
				var currentPath = pathArray[k];
				if(currentPath.indexOf("[") != -1) {
					// statefulObject is a List
					var index = currentPath.substring(1, currentPath.length-1);
					var plainObject = statefulObject["items"][index];
					statefulObject = statefulObject.get(index);
				} else {
					var plainObject = statefulObject[currentPath];
					statefulObject = statefulObject.get(currentPath);
				}
				dataBindingHelper._updatePlainValue(plainObject, pathArray, k+1, name, clonedPlainValue);
				if(!statefulObject) {
					return;
				}
			}
		}
	}
	
	dataBindingHelper._computePaths = function(bindingStr) {
		var sourcePathArray = bindingStr.split(".");
		var finalPathArray = [];

		for(var k=0; k<sourcePathArray.length; k++) {
			onePath = sourcePathArray[k];
			var positionOfLeftBracket = onePath.indexOf("[");
			if(positionOfLeftBracket != -1) {
				// name of the List
				finalPathArray.push(onePath.substring(0, positionOfLeftBracket));
				// index with bracket, e.g. [1]
				finalPathArray.push(onePath.substring(positionOfLeftBracket));				
			} else {
				finalPathArray.push(onePath);
			}
		}	
		return finalPathArray;
	}

	dataBindingHelper._updatePlainValue = function(plainObject, pathArray, index, name, value) {
		if(!plainObject) {
			return;
		}
		
		var target = plainObject;
		for(var k=index; k<pathArray.length; k++) {
			var currentPath = pathArray[k];
			if(currentPath.indexOf("[") != -1) {
			    // RTC PMR 219276 - PMR: 40425,057,649 
			    if(!target["items"]) return;

			    // target is a List
				var index = currentPath.substring(1, currentPath.length-1);
				target = target["items"][index];
			} else {
				target = target[currentPath];
			}
			if(!target) {
				return;
			}
		}
		target[name] = value;
	}
		
	return dataBindingHelper;
});
	