/*
Licensed Materials - Property of IBM
5725-C95
 Copyright IBM Corporation 2012. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

// Contains all utilities that Coach runtime supports multiple lists binding

define(["dojo/_base/array", "dojo/dom-class", "./CONSTANTS", "./domHelper", "./dataBindingHelper"], function(array, domClass, CONSTANTS, domHelper, dataBindingHelper) {
				
	var multiBindingListsHelper = new Object();
	
	var _triedToLoadMessage = false;
	
	// returns an array of binding str that represent a list and is different from the binding of specified coachView
	multiBindingListsHelper.analyzeRelatedListBindingStrs = function(coachView) {
		var results = [];		
		if(multiBindingListsHelper.isBindingObjectList(coachView)) {
			var dataBinding = dataBindingHelper.rtDataBinding._get(coachView.context.element);
			var noBracket = dataBinding.substring(0, dataBinding.length -2);	
	    	for(var domId in coachView._domIdToBindingListMap) {
	    		var listStr = coachView._domIdToBindingListMap[domId];
	    		if(noBracket != listStr && array.indexOf(results, listStr) == -1) {
	    			results.push(listStr);
	    		}
	    	}		    	
		}
		
		return results;
	}
	
	multiBindingListsHelper.isBindingObjectList = function(coachView) {
	    if(!!coachView.context.binding) {
			var bindingObject = coachView.context.binding.get("value");
			if(bindingObject instanceof com.ibm.bpm.coach.List) {
				return true;
			}
    	}
    	
    	return false;
	}
	
	
	multiBindingListsHelper.processMonitoredLists = function(coachView, engine) {
		if(!coachView.context.binding) {
			return;
		}
    	var myBindingList = coachView.context.binding.get("value");
    	if(myBindingList instanceof com.ibm.bpm.coach.List) {
    		var innerListShorterThanOuterList = false;
    		var innerListLongerThanOuterList = false;
    		var lengthOfMyBindingList = myBindingList.length();
	    	var monitoredList;
	    	for(var bindingStr in coachView._bindingStrToListMap) {
	    		monitoredList = coachView._bindingStrToListMap[bindingStr];
	    		var monitoredListLength = monitoredList.length(); 
	    		if(monitoredListLength > lengthOfMyBindingList) {
	    			innerListLongerThanOuterList = true;
	    		} else if(monitoredListLength < lengthOfMyBindingList) {
	    			innerListShorterThanOuterList = true;
	    		}
	    	}
	    	
	    	multiBindingListsHelper.clearWarning(coachView);
	    			    	
	    	if(innerListShorterThanOuterList || innerListLongerThanOuterList) {
	    		 multiBindingListsHelper.showWarning(coachView, innerListShorterThanOuterList, innerListLongerThanOuterList, engine);
	    	}
    	}
    }
	    
    multiBindingListsHelper.clearWarning = function(coachView) {
    	var warningDiv = document.getElementById(multiBindingListsHelper.getWarningDivId(coachView));
    	if(!!warningDiv) {
    		warningDiv.parentNode.removeChild(warningDiv);
    	}
    }
    
    multiBindingListsHelper.getWarningDivId = function(coachView) {
    	return coachView.context.element.id + CONSTANTS.COACH_VIEW_WARNING;
    }
    
    multiBindingListsHelper.showWarning = function(coachView, innerListShorterThanOuterList, innerListLongerThanOuterList, engine) {
    	
    	if(!engine.localizedMessages.coachNG_warning_outterListLongerThanInnerList) {
    		if(!_triedToLoadMessage) {
    			engine.forceLocalizedMessages();
    			_triedToLoadMessage = true;
    		} 
    	}
    	var warningDiv = document.createElement("div");
		domClass.add(warningDiv, CONSTANTS.COACH_VIEW_WARNING);
		warningDiv.id = multiBindingListsHelper.getWarningDivId(coachView);
		
		if(innerListShorterThanOuterList) {
			var warningMssage1 = engine.localizedMessages.coachNG_warning_outterListLongerThanInnerList;
			multiBindingListsHelper.appendOneWarning(warningDiv, warningMssage1);
		}
		if(innerListLongerThanOuterList) {
			var warningMssage2 = engine.localizedMessages.coachNG_warning_outterListShorterThanInnerList;
			multiBindingListsHelper.appendOneWarning(warningDiv, warningMssage2);
		}
		
	    coachView.context.element.appendChild(warningDiv);
    }
    
    multiBindingListsHelper.appendOneWarning = function(warningDiv, warningMsg) {
    	var wapperDiv = document.createElement("div");
		var warningTextNode = document.createTextNode(warningMsg);
		wapperDiv.appendChild(warningTextNode);
		warningDiv.appendChild(wapperDiv);
    }
    
	return multiBindingListsHelper;
});
	    