/*
Licensed Materials - Property of IBM
5725-C95
(C) Copyright IBM Corporation 2012, 2014. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

// This module handles data validation in caoch framework

define(["./dataBindingHelper", "./BPM_CONSTANTS", "./commonHelper", "./coachViewHelper", "dojo/_base/array"], function(dataBindingHelper, BPM_CONSTANTS, commonHelper, coachViewHelper, array) {
				
	var validationHelper = new Object();
	
	// only the public APIs of this module can access validationHelper properties which are specified here.
	validationHelper.initialize = function() {
		validationHelper.rawEvents = null;
		
		validationHelper.bindingToViewDomIdsMap = {};
		
		validationHelper.viewTree = {viewDomIdToTreeNodeMap: {}, rootNode: {}};
		
		validationHelper.lastListItemRemoved = {bindingListStr: null, removedItem: null};
		
		// transient info
		validationHelper.clearTransientInfo();
	}
	
	validationHelper.clearTransientInfo = function() {
		validationHelper.viewDomIdToViewMap = {};
	}
	
	validationHelper.validateAllViews = function(rawEvent, topLevelViews) {
		validationHelper.rawEvents = rawEvent;
	
		var viewDomIdToViewMap = validationHelper.viewDomIdToViewMap;
		var viewDomIdToTreeNodeMap = validationHelper.viewTree.viewDomIdToTreeNodeMap;
		var databindingToViewDomIdsMap = validationHelper.bindingToViewDomIdsMap;
		validationHelper._collectInfoForAllViews(topLevelViews, viewDomIdToViewMap, validationHelper.viewTree.rootNode, viewDomIdToTreeNodeMap, databindingToViewDomIdsMap);
		
		return validationHelper._process(rawEvent, databindingToViewDomIdsMap, viewDomIdToViewMap, viewDomIdToTreeNodeMap, true);
	}

	validationHelper.validateView = function(coachView) {
		var foundvalidationEvent = false;
		
		if(!validationHelper.rawEvents || validationHelper.rawEvents.length == 0) {
			// there is even no raw events, no nothing needs to do
			return foundvalidationEvent;
		}
		
		var treeNodeForView = null;
		
		// see if this coachView is a replacement of disposed view. If so, the validation event is already there, just need to make it up to date
		var myBinding = dataBindingHelper.rtDataBinding._get(coachView.context.element);
		var viewDomIds = validationHelper.bindingToViewDomIdsMap[myBinding];
		if(!!viewDomIds  && viewDomIds.length != 0) {
			var myLocationInfo = coachViewHelper.getViewIdMetaData(coachView);
			var myViewDomId =  coachView.context.element.id;
			// some other view has same binding as mine
			for(var k=0; k<viewDomIds.length; k++) {
				var treeNode = validationHelper.viewTree.viewDomIdToTreeNodeMap[viewDomIds[k]];
				if(!!treeNode) {
					if(coachViewHelper.hasSameStructure(myLocationInfo, treeNode.locationInfo)) {
						// coachView is the replacement for the view described by treeNode. So update that treeNode (and a few maps) to make sure it's up to date.
						validationHelper._updateTreeNode(treeNode, myViewDomId, myBinding, myLocationInfo, validationHelper.viewTree.viewDomIdToTreeNodeMap, validationHelper.bindingToViewDomIdsMap);
						treeNodeForView = treeNode;
						break;
					}
				}
			}		
		}
		
		if(!!treeNodeForView) {
			foundvalidationEvent = !!(treeNodeForView.validationEvent);
			
			// handle the subviews of this coachView
			for (var viewId in coachView.context.subview) {
				var subviews = coachView.context.subview[viewId];
				for(var divId in subviews) {
					var subViewScope = subviews[divId];
					var result = validationHelper.validateView(subViewScope);
					if(!foundvalidationEvent) {
						foundvalidationEvent = result;
					}					
				}
			}		
		} else {
			// this is a brand new coach view, need to check if it has validation errors
			foundvalidationEvent = validationHelper._validateView(coachView);
		}
		
		return foundvalidationEvent;
	}

	validationHelper.listItemRemoved = function(bindingListStr, removedIndex, removedItem) {

		if(!validationHelper.rawEvents || validationHelper.rawEvents.length == 0) {
			// there is even no raw events, no nothing needs to do
			return;
		}
		
		if(validationHelper.lastListItemRemoved.bindingListStr == bindingListStr && validationHelper.lastListItemRemoved.removedItem === removedItem) {
			// multiple views could bind to same bindingListStr. And those views may all try to tell validationHelper
			// that one item has been removed.
			// removedItem has been handled, so ignore it here.
			return;
		}
		
		validationHelper.lastListItemRemoved.bindingListStr = bindingListStr;
		validationHelper.lastListItemRemoved.removedItem = removedItem;

		// update validationHelper.rawEvents
		var invalidEventIndexs = [];
		for(var i=0; i<validationHelper.rawEvents.length; i++) {
			var errorBoPath = validationHelper.rawEvents[i].errorBOPath;
			var newErrorBoPath = validationHelper._updateBindingStrPerDelete(bindingListStr, removedIndex, errorBoPath);
			if(newErrorBoPath == null) {
				// list item that contains errorBoPath has been deleted. so remove this raw event too
				invalidEventIndexs.push(i);
			} else if(newErrorBoPath != errorBoPath) {
				validationHelper.rawEvents[i].errorBOPath = newErrorBoPath;
			}
		}	
		for(var k=invalidEventIndexs.length-1; k>=0; k--) {
			validationHelper.rawEvents.splice(invalidEventIndexs[k], 1);
		}
		
		// item has been deleted from list. As the result, existing information might be out-of-date, 
		// e.g. relativePath in computed validation events, the validation event that roots on deleted item, data in validationHelper.bindingToViewDomIdsMap etc.
		// the best way is to clean up, when new view is created, we evaluate it against the reserved (and updated) rawEvents.
		var cacheRawEvents = validationHelper.rawEvents;
		var cacheLastListItemRemoved = validationHelper.lastListItemRemoved;
		validationHelper.initialize();
		validationHelper.rawEvents = cacheRawEvents;
		validationHelper.lastListItemRemoved = cacheLastListItemRemoved;		
	}
	
	validationHelper._updateBindingStrPerDelete = function(bindingListStr, removedIndex, bindingToUpdate) {
		// bindingListStr is sth like a[0].b[1].c[]
		var strToMatch = bindingListStr.substring(0, bindingListStr.length-1);
		if(bindingToUpdate.indexOf(strToMatch) != -1) {
			var restPart = bindingToUpdate.substring(bindingListStr.length-1);
			var rightBracketPosition = restPart.indexOf("]");
			var index = restPart.substring(0, rightBracketPosition);
			if(index != "") {
				if(index == removedIndex) {
					return null;
				} else if(index > removedIndex) {
					var newBindingStr = strToMatch + (index-1) + restPart.substring(rightBracketPosition);
					return newBindingStr;
				}
			}			
		}
		
		return bindingToUpdate;
	}
	
	// validate the coachView against the raw events, prepare the validationEvent if exists
	validationHelper._validateView = function(coachView) {
	
		var myViewDomId = coachView.context.element.id;

		var rawEvent = validationHelper.rawEvents;
		
		var viewDomIdToViewMap = validationHelper.viewDomIdToViewMap;
		viewDomIdToViewMap[myViewDomId] = coachView;
		
		var viewDomIdToTreeNodeMap = validationHelper.viewTree.viewDomIdToTreeNodeMap;
		
		// insert into right place in validationHelper.viewTree
		var parentTreeNode = null;		
		var parentView = coachView.context.parentView();
		if(!!parentView) {
			parentTreeNode = viewDomIdToTreeNodeMap[parentView.context.element.id];
		}
		if(!parentTreeNode) {
			// can't find proper parent treeNode, let the rootNode to be parent
			parentTreeNode = validationHelper.viewTree.rootNode;
		}
			
		var databindingToViewDomIdsMap = new Object();
		validationHelper._collectInfoForView(coachView, databindingToViewDomIdsMap, viewDomIdToViewMap, parentTreeNode, viewDomIdToTreeNodeMap);
		
		return validationHelper._process(rawEvent, databindingToViewDomIdsMap, viewDomIdToViewMap, viewDomIdToTreeNodeMap, false);		
	}
			
	validationHelper._updateTreeNode = function(treeNode, newViewDomId, dataBinding, newLocationInfo, viewDomIdToTreeNodeMap, bindingToViewDomIdsMap) {
		var oldViewDomId = treeNode.domId;
		
		// update the domId of the treeNode
		treeNode.domId = newViewDomId;
		
		// coachView is a new one, we need to deliver the validation event to this coachView again if it's already done
		if(treeNode.isHandled) {
			treeNode.isHandled = false;
		}
		
		// update validationEvent
		validationHelper._updateIdInValidationEvent(oldViewDomId, newViewDomId, treeNode.validationEvent);
		
		// update the domId referenced by the parent views
		var tempNode = treeNode.parent;
		while(!!tempNode) {
			validationHelper._updateIdInValidationEvent(oldViewDomId, newViewDomId, tempNode.validationEvent, true);
			tempNode = tempNode.parent;
		}
		
		// update locationInfo
		treeNode.locationInfo = newLocationInfo;
		
		// update validationHelper.viewTree.viewDomIdToTreeNodeMap
		viewDomIdToTreeNodeMap[newViewDomId] = viewDomIdToTreeNodeMap[oldViewDomId];
		delete viewDomIdToTreeNodeMap[oldViewDomId];
		
		// update validationHelper.bindingToViewDomIdsMap
		var domIdsOnOneBinding = bindingToViewDomIdsMap[dataBinding];
		validationHelper._replaceIdInArray(oldViewDomId, newViewDomId, domIdsOnOneBinding);
	}
	
	validationHelper._updateIdInValidationEvent = function(oldId, newId, event, updateSubscriptionError) {
		if(!event) {
			return;
		}
		
		if(!!event.errors) {
			for(var k=0; k<event.errors.length; k++) {
				var domIdsInOneError = event.errors[k].view_dom_ids;
				validationHelper._replaceIdInArray(oldId, newId, domIdsInOneError);
			}
		}
		
		if(updateSubscriptionError == true) {
			if(!!event.errors_subscription) {
				for(var k=0; k<event.errors_subscription.length; k++) {
					if(event.errors_subscription[k].view_dom_id == oldId) {
						event.errors_subscription[k].view_dom_id = newId;
					}
				}
			}
		}
	}
	
	validationHelper._replaceIdInArray = function(oldId, newId, idArray) {
		index = array.indexOf(idArray, oldId);
		if(index != -1) {
			idArray[index] = newId;
		}
	}
	
	// prepare the validation events if exists
	validationHelper._process = function(rawEvent, databindingToViewDomIdsMap, viewDomIdToViewMap, viewDomIdToTreeNodeMap, reportUnResolvedErrorBoPath) {	
		var foundValidationEvent = false;
		
		// map between the dom id of views and their list of error messages.
		var viewDomIdToErrorMessagesMap = new Object();			

		for(var i=0; i<rawEvent.length; i++) {
			var foundOneView = false;
			var errorBoPath = rawEvent[i].errorBOPath;
			var errorMsg = rawEvent[i].errorMessage;
			var pathArray = errorBoPath.split(".");
			// path always start with "local."
			var databindingToTry = pathArray[0];
			
			var relatedErrors = new Array();
			for(var k=1; k<pathArray.length; k++) { 
				var relativePath = validationHelper.computeRelativePath(pathArray, k+1);
				var endPosition = pathArray[k].indexOf("[");
				if(endPosition != -1 && pathArray[k].indexOf("[]") == -1) {
					// if pathArray[k] is "abc[number]", we need to check "abc[]" as well
					var otherDatabindingToTry = databindingToTry + "." + pathArray[k].substring(0, endPosition) + "[]";
					
					if(!!databindingToViewDomIdsMap[otherDatabindingToTry]) {
						var newRelativePath = pathArray[k].substring(endPosition);
						if(relativePath != "") {
							newRelativePath += "." + relativePath;
						}
						var viewDomIds = databindingToViewDomIdsMap[otherDatabindingToTry];
						validationHelper.handleOneRelativePath(newRelativePath, errorMsg, viewDomIds, viewDomIdToViewMap, viewDomIdToTreeNodeMap, viewDomIdToErrorMessagesMap, relatedErrors);
						if(!foundOneView) {
							foundOneView = true;
						}
					}				
				}
				databindingToTry += "." + pathArray[k];
				if(!!databindingToViewDomIdsMap[databindingToTry]) {
					var viewDomIds = databindingToViewDomIdsMap[databindingToTry];
					validationHelper.handleOneRelativePath(relativePath, errorMsg, viewDomIds, viewDomIdToViewMap, viewDomIdToTreeNodeMap, viewDomIdToErrorMessagesMap, relatedErrors);
					if(!foundOneView) {
						foundOneView = true;
					}
				}				
			}
			
			if(!!reportUnResolvedErrorBoPath && !databindingToViewDomIdsMap[errorBoPath]) {
				// no view binds to errorBoPath
		    	var message = "validationHelper._process(), Warning: No view binds errorBoPath: " + errorBoPath;
		    	commonHelper.consoleLog(message, commonHelper.WARN);
			}
			
			foundValidationEvent = foundValidationEvent || foundOneView;
		}
			
		// handle subscription
		for(errorViewDomId in viewDomIdToErrorMessagesMap) {
			var errorView = viewDomIdToViewMap[errorViewDomId];
			var parentView = errorView.context.parentView();
			while(!!parentView) {
				if(parentView.context._validationSubscribed) {
					var parentViewDomId = parentView.context.element.id;
					if (!!(viewDomIdToTreeNodeMap[parentViewDomId])) {
    					var parentViewFineEvent = viewDomIdToTreeNodeMap[parentViewDomId].validationEvent;
    					if(!parentViewFineEvent || !validationHelper.contains(parentViewFineEvent, errorViewDomId)) {
    						if(!parentViewFineEvent) {
    							parentViewFineEvent = new Object();
    							parentViewFineEvent.type = BPM_CONSTANTS.VALIDATION_EVENT_TYPE_ERROR;
    							viewDomIdToTreeNodeMap[parentViewDomId].validationEvent = parentViewFineEvent;
    							// when there is validationEvent, we also collect 
    							viewDomIdToTreeNodeMap[parentViewDomId].locationInfo = coachViewHelper.getViewIdMetaData(parentView);
    						} 
    						if(!parentViewFineEvent.errors_subscription) {
    							parentViewFineEvent.errors_subscription = new Array();
    						}
    						
    						var oneEntry = new Object();
    						oneEntry.view_dom_id = errorViewDomId;
    						oneEntry.messages = viewDomIdToErrorMessagesMap[errorViewDomId];
    						parentViewFineEvent.errors_subscription.push(oneEntry);
    					}
					}
				}
				parentView = parentView.context.parentView();					
			}
		}
		
		return foundValidationEvent;	
	}
	
	validationHelper.handleOneRelativePath = function(relativePath, errorMsg, viewDomIds, viewDomIdToViewMap, viewDomIdToTreeNodeMap, viewDomIdToErrorMessagesMap, viewDomIdToSameErrorMap) {

		for(var viewCounter=0; viewCounter<viewDomIds.length; viewCounter++) {
			var viewDomId = viewDomIds[viewCounter];
			var aView = viewDomIdToViewMap[viewDomId];

			var fineEvent = viewDomIdToTreeNodeMap[viewDomId].validationEvent; 			
			if(!fineEvent) {
				fineEvent = new Object();
				fineEvent.type = BPM_CONSTANTS.VALIDATION_EVENT_TYPE_ERROR;
				viewDomIdToTreeNodeMap[viewDomId].validationEvent = fineEvent;
				// when there is validationEvent, we also collect 
				viewDomIdToTreeNodeMap[viewDomId].locationInfo = coachViewHelper.getViewIdMetaData(aView);
			}
			if(!fineEvent.errors) {
				fineEvent.errors = new Array();
			}
			var error = validationHelper.getErrorByPath(fineEvent.errors, relativePath, errorMsg);
			error.view_dom_ids.push(viewDomId);
			
			var parentView = aView.context.parentView();
			while(!!parentView) {
				var parentViewDomId = parentView.context.element.id;
				var sameErrorInParent = viewDomIdToSameErrorMap[parentViewDomId];
				if(!!sameErrorInParent) {
					// parentView has same error as aView, so add id of aView to the error of parentView
					sameErrorInParent.view_dom_ids.push(viewDomId);
				}
				parentView = parentView.context.parentView();										
			}
			viewDomIdToSameErrorMap[viewDomId] = error;
			
			if(!!viewDomIdToErrorMessagesMap[viewDomId]) {
				viewDomIdToErrorMessagesMap[viewDomId].push(errorMsg);
			} else {
				var errorMessages = new Array();
				errorMessages.push(errorMsg);
				viewDomIdToErrorMessagesMap[viewDomId] = errorMessages;
			}					
		}				
	}
	
	validationHelper.computeRelativePath = function(pathArray, pathArrayStartPosition) {
		var relativePath = "";
		for(var j=pathArrayStartPosition; j<pathArray.length; j++) {
			if(relativePath != "") {
				relativePath += "." + pathArray[j];
			} else {
				relativePath = pathArray[j];
			}						
		}
		
		return relativePath;
	}
	
	// check if validationEvent.errors already contains specified viewDomId
	validationHelper.contains = function(validationEvent, viewDomId) {
		if(!!validationEvent.errors) {
			for(var k=0; k<validationEvent.errors.length; k++) {			
				if(array.indexOf(validationEvent.errors[k].view_dom_ids, viewDomId) != -1) {
					return true;
				}
			}
		}
		return false;
	}
	
	// if there's no error for specified relativePath, create one and add it to the error array
	validationHelper.getErrorByPath = function(errors, relativePath, errorMessage) {
		for(var k=0; k<errors.length; k++) {			
			if(errors[k].binding === relativePath) {
				return errors[k];
			}
		}
		
		var error = new Object();
		error.binding = relativePath;
		error.message = errorMessage;
		error.view_dom_ids = new Array();
		
		errors.push(error);
		
		return error;
	}
	
	validationHelper._collectInfoForAllViews = function(topLevelViews, viewDomIdToViewMap, viewTreeRootNode, viewDomIdToTreeNodeMap, databindingToViewDomIdsMap) {
		var bindingStrToListMap = new Object();
		for (var viewId in topLevelViews) {
			var viewScope = topLevelViews[viewId];
			validationHelper._collectInfoForView(viewScope, databindingToViewDomIdsMap, viewDomIdToViewMap, viewTreeRootNode, viewDomIdToTreeNodeMap);
		}
	}
	
	// crete the treeNode and collect binding info for hte viewScope and its descendant views
	validationHelper._collectInfoForView = function(viewScope, databindingToViewDomIdsMap, viewDomIdToViewMap, parentTreeNode, viewDomIdToTreeNodeMap) {
		var treeNode = validationHelper._createTreeNode(viewScope, parentTreeNode, viewDomIdToTreeNodeMap);		
		validationHelper._collectBindingForOneView(viewScope, databindingToViewDomIdsMap);
		
		viewDomIdToViewMap[viewScope.context.element.id] = viewScope;

		if (!!viewScope.context.subview) {
			for (var viewId in viewScope.context.subview) {
				var subviews = viewScope.context.subview[viewId];
				for(var divId in subviews) {
					var subViewScope = subviews[divId];
					validationHelper._collectInfoForView(subViewScope, databindingToViewDomIdsMap, viewDomIdToViewMap, treeNode, viewDomIdToTreeNodeMap);
				}
			}		
		}			
	}
	
	validationHelper._collectBindingForOneView = function(coachView, databindingToViewDomIdsMap) {
		var bindingStrArray = coachView._getBindingStrings();
		for(var k=0; k<bindingStrArray.length; k++) {
			var bindingStr = bindingStrArray[k];
			var coachViewDomIdArray = databindingToViewDomIdsMap[bindingStr];
			if(!coachViewDomIdArray) {
				coachViewDomIdArray = new Array();
				databindingToViewDomIdsMap[bindingStr] = coachViewDomIdArray;
			}
			coachViewDomIdArray.push(coachView.context.element.id);		
		}
	}
	
	validationHelper._createTreeNode = function(viewScope, parentTreeNode, viewDomIdToTreeNodeMap) {
		var viewDomId = viewScope.context.element.id;
		var treeNode = {"viewId": viewScope.context.viewid, 
						"domId": viewDomId, 
						"isHandled": false};
		treeNode.parent = parentTreeNode;
		if(!parentTreeNode.firstChild) {
			parentTreeNode.firstChild = treeNode;
		} else {
			parentTreeNode.lastChild.next = treeNode;
		}
		parentTreeNode.lastChild = treeNode;
		
		viewDomIdToTreeNodeMap[viewDomId] = treeNode;
		
		return treeNode;
	}
		
	
	validationHelper.initialize();
	
	return validationHelper;
});
	