/*
 * Copyright (C) 2006 Lombardi Software
 * http://www.lombardi.com/
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of Lombardi
 * Software ("Confidential Information"). You shall not disclose such 
 * Confidential Information and shall use it only in accordance with the terms
 * of the license agreement and/or Non-Disclosure Agreement you entered into 
 * with Lombardi Software. No rights or licenses are granted with respect to
 * the Confidential Information except as expressly set forth in such License
 * Agreement and/or Non-Disclosure Agreement. 
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL LOMBARDI
 * SOFTWARE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * JavaScript file, contains code of Add/Update Document Form.
 * 
 * Created by Sergey Rassokhin, 9/5/2007 
 */
 
 /**
  * Class wrapps <form/> element. Allow more simply creating and modificating
  * input fields. 
  *
  * @class 
  * @constructor  
  */

var validationMessage;
var docNameMaxLength = 50; // This needs to be consistent with Struts validation rule.

 function Form(prefix) {
 	// prefix for all IDs of fields (required for construct object) 
 	this.prefix = prefix;
 	// root DOM element of control
 	this.root = null;
 	// form action URL
 	this.actionUrl = null;
 	// onclose event handler - calls when close button pressed
 	this.onclose = function() {};
 	// onsubmit event handler - calls before form submit
 	this.onsubmit = function() {}; 	
 	// Map object = { "<field_name>" : { 
 	//					"title" : "<title>",
 	// 					"defaultValue" : "<default_value>", 
 	// 					"validator" : validator 
 	// 				  } 
 	// 				} 	 	 	
 	this.fields = new Object();
 	this.container = null;
     this.validationMessage='Empty message';
     

 }
 
 /** 
  * Initializes element. Creates <form/> element and place into it <iframe/> 
  * (for submitting), <table/> elements and "ok"/"cancel" buttons. 
  */
 Form.prototype.init = function() {
 	var obj = this;
 	// create table
 	var table = document.createElement("table");
	table.className = "docAttachmentTableNoLines";
	table.style.width = "100%";
	table.style.margin = "0px";
	table.style.marginBottom = "7px";
	table.style.borderCollapse = "separate";
	table.cellSpacing = "3px";	
	var tbody = document.createElement("tbody");	
	table.appendChild(tbody);
	this.tbody = tbody;
 	// create buttons
 	var ok = createButton1(documentAttachment_button_ok, function() {
 		obj.submit();
 	});
        ok.className += " twDocAttachmentOkButton";
        var cancel = createButton1(documentAttachment_button_cancel, function(){ 		
 		obj.onclose();
 	});
        cancel.className += " twDocAttachmentCancelButton";
        // create form
 	var form = document.createElement("div");
 	this.root = form;
 	// append all elements
 	form.appendChild(table);
 	form.appendChild(ok);
 	form.appendChild(cancel); 	
 }
 
 /**
  * Adds row to embedded table.
  * 
  * @private
  * @argument {string} title - title, which will be shown at left column
  * @argument {DOM element} field - element, which will be shown at rights column
  * @argument {boolean} visible - initial visible state of row
  * @argument {string} id - <tr/> element id
  */
 Form.prototype.addRow = function (title, field, visible, rowId) {
 	var tr = document.createElement("tr");
 	tr.id = rowId;
	var td = document.createElement("td");
	td.innerHTML = title;
	td.align = "right";
	td.width = "50px";
	tr.appendChild(td);
	var td = document.createElement("td");
	td.appendChild(field);
	tr.appendChild(td);
	tr.style.display = visible ? "" : "none";
	this.tbody.appendChild(tr);
 }
 
 /** 
  * Generates field's id, id = %prefix% + "field_" + name. Prefix is generated 
  * during constructor execution and it is unique for each instance of Form 
  * object.
  */
 Form.prototype.getFieldId = function(name) {
 	return this.prefix + "_field_" + name;
 }
 
 /** 
  * Generates row's id, id = %prefix% + "tr_" + name. Prefix is generated 
  * during constructor execution and it is unique for each instance of Form 
  * object.
  */
 Form.prototype.getRowId = function(name) {
 	return this.prefix + "_tr_" + name;
 }
 
 /**
  * Adds hidden field into form (<input type="hidden" .../>).
  */
 Form.prototype.addHiddenField = function (name, defaultValue) {
 	var element = createInput("hidden", name, defaultValue);
 	element.id = this.getFieldId(name);
 	this.root.appendChild(element);
 	this.saveField(name, name, defaultValue, null); 	 	
 }
 
 /**
  * Adds input text field into form (<input type="text" .../>).
  */
 Form.prototype.addInputTextField = function (name, defaultValue, title, visible, width) {
 	this.addTextField(name, defaultValue, title, visible, null);
 }
 
 /**
  * Adds input text field into form (<input type="text" .../>).
  */
 Form.prototype.addInputTextField = function (name, defaultValue, title, visible, width, validator) {
 	var input = document.createElement("input");
 	input.type = "text";
 	input.name = name;
 	input.id = this.getFieldId(name);
 	input.value = defaultValue; 	 
 	input.style.fontSize = "10px";
 	input.style.width = width;
 	this.addRow(title, input, visible, this.getRowId(name));
 	this.saveField(name, title, defaultValue, validator);
 }
 
 /**
  * Adds upload file field into form (<input type="file" .../>).
  */
 Form.prototype.addInputFileField = function (name, defaultValue, title, visible, width) {
 	this.addInputFileField(name, defaultValue, title, width, null); 	
 }
 
 /**
  * Adds upload file field into form (<input type="file" .../>).
  */
 Form.prototype.addInputFileField = function (name, defaultValue, title, visible, width, validator) {
 	var input = document.createElement("input");
 	input.type = "file";
 	input.contenteditable = false;
 	input.name = name;
 	input.id = this.getFieldId(name);
 	input.value = defaultValue; 	 
 	input.style.fontSize = "10px"; 	
	input.size = width;
 	this.addRow(title, input, visible, this.getRowId(name));
 	this.saveField(name, title, defaultValue, validator);
 }
 
 /**
  * Adds text field into form (<span/>).
  */
 Form.prototype.addTextField = function (name, defaultValue, title, visible) {
 	var text = document.createElement("span");
 	text.innerHTML = defaultValue;
 	text.id = this.getFieldId(name);
 	this.addRow(title, text, visible, this.getRowId(name));
 	this.saveField(name, title, defaultValue, null);
 }
 
 /**
  * Adds select input element (<input type=select".../>)  
  */ 
 Form.prototype.addSelectField = function (name, defaultValue, title, width, visible, values, onchange) {
 	var select = document.createElement("select");
 	select.name = name; 	
 	select.id = this.getFieldId(name); 	
 	for (val in values) {
 		select.appendChild(createOption(resourceBundle[val], values[val], false));
 	} 	
 	select.multiselect = "false";
	select.style.fontSize = "10px";	
	select.onchange = onchange;
 	this.addRow(title, select, visible, this.getRowId(name)); 	
 	this.saveField(name, title, defaultValue, null);
 }
 
 /** Sets row which contains specfied field visible/not visible */
 Form.prototype.setFieldVisible = function(name, visible) {
 	var tr = document.getElementById(this.getRowId(name));
 	tr.style.display = visible ? "" : "none";
 }

/**
  * Returns true if field is visible and false in other case.
  */
 Form.prototype.isFieldVisible = function(name) {
 	var field = document.getElementById(this.getRowId(name));
 	if (field) {
 		return field.style.display != "none"; 
 	}
 	return false;
 }

/** Sets value for specified field. */
 Form.prototype.setFieldValue = function(name, value) {
 	var field = document.getElementById(this.getFieldId(name)); 	
 	if (field.value != null) {
 		// if field can be editable (field is a input or select element)
 		field.value = value; 		
 	} else {
 		if (field.innerText != null && field.innerText != "") {
			field.innerText = value; 			
 		} else {
 			field.innerHTML = value;
 		} 		  		 		 		 	
 	} 	
 }
 
 /** 
  * Returns value of specified field. If the field is a <input/> or <select/> 
  * it returns element.value, in other case element.innerHTML. 
  */
 Form.prototype.getFieldValue = function(name) {
 	var field = document.getElementById(this.getFieldId(name));
 	if (field.value != null) {
 		return field.value;
 	} 
 	return (field.innerText != null) ? field.innerText : field.innerHTML; 	
 }

String.prototype.format = function() {
    var args = arguments;

    return this.replace(/\{(\d+)\}/g, function() {
        return args[arguments[1]];
    });
};

  /** Submits this form. */
 Form.prototype.submit = function() {
 	// First validate form before submitting
 	for (field in this.fields) {
		var validator = this.fields[field].validator;
		if (validator != null && this.isFieldVisible(field)) {
			var value = this.getFieldValue(field);
			if (field != "docName") { // validation for non-Title fields.
	                        var right = validator(value);
        	                if (!right) {
	                                var f = this.fields[field];
	                                alert(this.validationMessage.format( f.title ,value) );
	                                return;
	                        }
			} else {
				// Validate docName length and display a different message.
                                var right = validator(value);
                                if (!right) {
                                        var f = this.fields[field];
                                        alert(documentAttachment_docNameMaxLength.format(f.title, docNameMaxLength) );
                                        return;
                                }
				// If docName is not specified, validate filename.
				if (value == null || value == "") {
					var filepath = this.getFieldValue("fileContent");
					var filename = filepath.match(/(.*)[\/\\]([^\/\\]+\.\w+)$/);
					if (filename == null)
						filename = filepath;
					else
						filename = filename[2];
					var right = validator(filename);
	                                if (!right) {
        	                                var f = this.fields[field];
						var file = this.fields["fileContent"];
	                                        alert(documentAttachment_fileNameMaxLength.format(file.title, f.title, docNameMaxLength) );
	                                        return;
	                                }
				}
			}
			var right = validator(value);
			if (!right) {
				var f = this.fields[field];      
				alert(this.validationMessage.format( f.title ,value) );
				return;
			}
		}	
	}		
	this.onsubmit();
	var helperForm = document.getElementById("helperForm");
	if (helperForm != null) {
		helperForm.action = this.actionUrl;
		helperForm.appendChild(this.root);
		helperForm.submit();
                helperForm.reset();
                this.container.appendChild(this.root);
	}
 }

 /**
  * Saves field parameters for further looking for them by field's name.
  */
 Form.prototype.saveField = function(name, title, defaultValue, validator) {
 	var o = new Object();
 	o.title = title;
 	o.defaultValue = defaultValue;
 	o.validator = validator;
 	this.fields[name] = o;	
 }
 
 /** 
  * Sets field's values to it's default states. Default states are assigned with 
  * fields when "add some field" method is called.
  */
 Form.prototype.clear = function() {
	for (field in this.fields) {
		this.setFieldValue(field, this.fields[field].defaultValue);	
	}
 }
 
 Form.prototype.showAll = function() {
 	try {
	for (field in this.fields) {
		alert(field + "," + this.getFieldValue(field));	
	}	} catch(e) {
		alert(e);
	}
 }
 
 // Validators. All validator-functions have a following signature:
 // function <Name>Validator(value), returns boolean value that indicates
 // valid status of given value.
 
 function URLValidator(value) {
	if(value && (value.match(/^(http|ftp|https|sftp)\:\/\/\w+([\.\-]\w+)*(\:\d+)?([\w\/\.\-\?\&\%\#\=\@\$]*)*\/?$/i) ||
    	 value.match(/^mailto\:\w+([\.\-]\w+)*\@\w+([\.\-]\w+)*\.\w{2,4}$/i))){
    	return true;
  	} else {
    	return false;
  	}
 }
 
 function FILEValidator(value) {
 	return value != null && value != "";
 }

 function DocNameValidator(value) {
	return value.length <= docNameMaxLength;
 }


