// BEGIN COPYRIGHT
//*************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-A09
// Copyright IBM Corporation 2010. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
//*************************************************************************
//END COPYRIGHT

function DocumentViewerControl(root, id) {
  this.rootElement = root;
  this.controlId = id;

  this.iframe = null;
  this.linkDiv = null;
  this.link = null;

  this.instanceId = null;
  this.portalPrefix = "";
  this.userId = null;

  this.associationType = 0;
  this.associationControlId = "";

  this.connectionType = 0;

  this.byInstance = false;
  this.matchingMode = "";
  this.matchingProperties = null;

  this.snapshotId = "";
  this.processId = "";
  this.connectionUrl = "";
  this.connectionRepoName = "";
  this.connectionUsername = "";
  this.passwordMode = 0;
  this.freeTextPassword = "";
  this.password = "";

  this.remoteMatchingMode = "";
  this.itemClassName = "";
  this.remoteMatchingProperties = null;

  this.displayUrl = "";

}

DocumentViewerControl.prototype.setUrl = function(url) {
  this.iframe.src = url;
  this.link.href = url;
}

DocumentViewerControl.prototype.loadDocument = function() {
  if (this.associationType != 0) {
    return;
  }

  switch (this.connectionType) {
    // Lombardi documents
    case 0:
      var args = "";
      args += ("&bpdInstanceId=" + this.instanceId);
      args += ("&coachdesigner=false");
      if (this.matchingMode != "None") {
        for (prop in this.matchingProperties) {
          if (prop) {
            args += ("&docprop_" + prop + "=" + this.matchingProperties[prop]);
          }
        }
        args += ("&matching=" + this.matchingMode);
      }

      args += ("&docMatching=" + !this.byInstance);

      var url = this.portalPrefix + "/jsp/remotedocument?operation=ajax_loadTwDocument" + args;
      this.setUrl(url);
            
      break;
    // ECM documents
    case 1:
      var realPassword = this.passwordMode == 0 ? this.freeTextPassword : this.password;

      var args = "";
      for (var prop in this.remoteMatchingProperties) {
        if (prop) {
          args += ("&docprop_" + prop + "=" + this.remoteMatchingProperties[prop]);
          args += ("&selprop_" + prop.split('(')[0] + "=");
        }
      }
      args += ("&matching=" + this.remoteMatchingMode);

      args += ("&snapshotId=" + this.snapshotId);
      args += ("&processId=" + this.processId);
      args += ("&url=" + this.connectionUrl);
      args += ("&repositoryName=" + this.connectionRepoName);
      args += ("&username=" + this.connectionUsername);
      args += ("&password=" + realPassword);

      args += ("&itemClassName=" + this.itemClassName);

      var url = this.portalPrefix + "/jsp/remotedocument?operation=ajax_getFirstMatch" + args;

      this.setUrl(url);
      break;
    // URL display
    case 2:
      this.setUrl(this.displayUrl);
      break;
    default:
      window.alert("Unknown connection type: " + this.connectionType + " (valid values are 0, 1, 2)");
  }
}

DocumentViewerControl.prototype.init = function() {
  var iframeElement = document.createElement("iframe");
  iframeElement.className = "twControl twDocViewerFrame";
  iframeElement.src = "about:blank";

  this.iframe = iframeElement;
  this.rootElement.appendChild(this.iframe);

  var linkElement = document.createElement("div");
  linkElement.className = "twDocViewerLink";

  var link = document.createElement("a");
  link.href = "about:blank";
  link.target = "_blank";
  link.innerHTML = "open in new window";
  linkElement.appendChild(link);
  this.link = link;

  this.linkDiv = linkElement;
  this.rootElement.appendChild(this.linkDiv);
}
