// BEGIN COPYRIGHT
//*************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-A09
// Copyright IBM Corporation 2010. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
//*************************************************************************
//END COPYRIGHT 

//
//	Num Formating: Rule of precedence
//	
//	Format Selection
//	
//	1) Section Separator ";" 
//		0) 	nop 
//		1-n)	devide into sections, then pick the one fitting the sign of the number 
//		
//	
//	Number Scaling
//	
//	2) Percentage
//		0) 	nop
//		1-n)	multiply once by 100
//		
//	
//	3) Scaling: n-times a comma directly after the decimal point
//		0) 	nop 
//		1-n) 	scale the number by n: divide number n times by 1000.
//
//  3a) Scientific notation
//	
//	
//	4) Decimal seperator "."
//		0) 	cut off the rest using math.round()
//		1) 	determine position in string, cut of the rest 
//		2-n) 	if more than one exist, use the one on the far left (for now)?
//	
//	
//	(-> Number  to String)
//	
//	String formatting options
//
//	5)	Digit placeholder ("#") | Zero placeholder ("0") | Padding placeholder ("?")  | Literals | 
//
//  6)  Grouping
//	

function NumberFormatter() {
    // Defines
    this.SECTION_SEPARATOR = ";";
    this.PERCENT_INDICATOR = "%";
    this.DECIMAL_INDICATOR = ".";
    this.DIGIT_PLACEHOLDER = "#";
    this.ZERO_PLACEHOLDER = "0";
    this.PADDING_PLACEHOLDER = "?";
    this.GROUPING_DELIMITIER = ",";
    this.MINUS_SIGN = "-";
    this.EXPONENTIAL_CHAR = "E";
    // Member fields
    this.strOutRemainder = "";
    this.strOutMajor = "";
    this.currentTemplate = "";
    this.templateString = "";
    this.formattedNumber = "";
    this.valueAsString = "";
    this.value = NaN;
    this.templateString = "";
    this.templates = [];
    this.exponentialDigits = 0;
    this.exponentialSignTemplate = "";
    this.exponentialDigitsTemplate = "";
    // reusable RegExps
    // Scaling : /(\,+)\./g
    this.scalingRegExp = new RegExp("(\\" + this.GROUPING_DELIMITIER + "+)\\" + this.DECIMAL_INDICATOR, "g");
    // the decimal dot: /\./g
    this.dotsRegExp = new RegExp("\\" + this.DECIMAL_INDICATOR, "g");
    // Grouping: /(#|0),(#|0)/g
    var reGroup = "(" + this.DIGIT_PLACEHOLDER + "|" + this.ZERO_PLACEHOLDER + ")";
    this.groupingRegExp = new RegExp(reGroup + "," + reGroup, "g");
    // Find all commas (grouping char)  /,/g 
    this.groupCharsRegExp = new RegExp(this.GROUPING_DELIMITIER, "g");
    // any Placeholder for a digit  (\\#|0|\\?)  
    this.anyDigitRegExp = new RegExp("(\\" + this.DIGIT_PLACEHOLDER + "|\\" + this.ZERO_PLACEHOLDER + "|\\" + this.PADDING_PLACEHOLDER + ")", "g");
    // RegExp to find the far left digit in the String   (\d{1})  
    this.leftmostDigitRegExp = new RegExp("(\\d{1})");
    // RegExp to find scientific notation indicator  E0 E+0 E-0 e0 e+0 e-0 = /(E)(\+|-)?(0+)/i
    this.scientificRegExp = new RegExp("(" + this.EXPONENTIAL_CHAR + ")(\\+|-)?(" + this.ZERO_PLACEHOLDER + "+)", "i");
    // RegExp to find Number placeholders digits after the dot /\.((#|0)\?*)/
    this.digitsAfterDot = new RegExp("\\.((\\#|0)+\\?*)");
    // RegExp to parse a exponential number into groups /(\d+)\.(\d+)e(\+|-){1}(\d+)/i
    this.exponentialRegExp = new RegExp("(\\d+)\\.(\\d+)e(\\+|-){1}(\\d+)", "i");
}

NumberFormatter.prototype.format = function ( /*String*/ value, /*String*/ template, /*Boolean*/ isNumeric) {
    if (template == null || template == "") {
        return value;
    }
    if (value == null || value == "") {
        value = 0;
    }
    // reset member fields
    this.strOutRemainder = "";
    this.strOutMajor = "";
    this.currentTemplate = "";
    this.templateString = "";
    this.valueAsString = "";
    this.value = NaN;
    this.templateString = "";
    this.formattedNumber = "";
    this.templates = [];
    this.isNegative = 0;
    this.exponentialDigits = 0;
    this.exponentialSignTemplate = "";
    this.exponentialDigitsTemplate = "";
    // Do some inits/checkings ...
    if ((typeof value != "string") && (typeof value != "number")) {
        return value;
    }
    this.valueAsString = value + "";
    this.value = parseFloat(value);
    if (isNaN(this.value)) {
        // no formatting if no number
        return undefined;
    }
    this.templateString = template;
    if (this.value < 0) {
        // keep info about sign
        this.isNegative = true;
        this.value = Math.abs(this.value);
    }
    //////////////////////////////////////
    // 1) Determine section separator ";"
    //////////////////////////////////////
    this._processSections();
    // determine position of decimal dot
    var firstDot = this.currentTemplate.indexOf(this.DECIMAL_INDICATOR);
    ///////////////////
    // Number Scaling
    ///////////////////
    // 2) Percentage
    //    0) 	nop
    //    1-n)	multiply only once by 100 -> NOP for scaling
    if (this.currentTemplate.indexOf(this.PERCENT_INDICATOR) > -1) {
        this.value = this.value * 100;
    }
    // 3) Scaling: n-times a comma directy after the decimal point
    //		0) 	nop 
    //		1-n) 	scale the number by n: divide number n times by 1000.
    var matches = this.currentTemplate.match(this.scalingRegExp);
    if (matches != null) {
        // scale down
        var power = (matches[matches.length - 1].length) - 1;
        this.value = this.value / Math.pow(1000, power);
        // remove scaling commas from template, but leave "." if there are digits on the right side of "."
        if (firstDot == (this.currentTemplate.length - 1)) {
            // dot is the last char in template -> remove it 
            this.currentTemplate = this.currentTemplate.replace(this.scalingRegExp, "");
            firstDot = -1; // dot was no real fraction indicator
        }
        else {
            // leave the dot
            this.currentTemplate = this.currentTemplate.replace(this.scalingRegExp, this.DECIMAL_INDICATOR);
        }
        // Position of dot may have moved, recalc 
        firstDot = this.currentTemplate.indexOf(this.DECIMAL_INDICATOR);
    }
    // 3a) Scientific notation
    //     0) NOP
    //     1-n) take the far left only, leave the others
    matches = this.currentTemplate.match(this.scientificRegExp);
    if (matches != null) {
        // matching groups are as follows: 
        // [0] : complete group, e.g. "E-0"
        // [1] : exponential sign, = "E" or "e"
        // [2] : sign for the exponent: "+", "-", or null
        // [3] : amount of digits to be displayed, e.g. "00", at least "0"
        if (matches[3]) {
            this.exponentialDigits = matches[3].length;
            this.exponentialDigitsTemplate = matches[3];
        }
        this.exponentialChar = matches[1];
        // remember sign		
        this.exponentialSignTemplate = matches[2];
        // replace precision of exponent to be not formatted
        // e.g. replace  777533E03 with 7775.33E
        this.currentTemplate = this.currentTemplate.replace(this.scientificRegExp, "$1");
        // Position of dot may have moved, recalc 
        firstDot = this.currentTemplate.indexOf(this.DECIMAL_INDICATOR);
    }
    // 4) Decimal separator "."
    //		0) 	cut off the rest using math.round()
    //		1) 	// nop so far, (not sure: cut of the rest using toFixed?)
    //		2-n) 	if more than one exist, use the one on the far left (for now)?
    if (firstDot == -1) {
        this.value = Math.round(this.value);
    }
    /////////////////////////////////	
    // 5) String formatting options
    /////////////////////////////////
    //		Digit placeholder ("#") | Zero placeholder ("0") | Padding placeholder ("?")  | Literals 
    var nMajor = Math.floor(this.value);
    // we can not use subtraction for getting remainder because of precision loss
    var numStr = this.value + "";
    var dotPos = numStr.indexOf(".");
    var nRemainder = (dotPos > -1) ? "0." + numStr.substring(dotPos + 1) : 0;
    var exp = null;
    var exponentialSign = "";
    var strExponent = "";
    if (this.exponentialDigits > 0) {
        // determine number of digits after the dot
        var digitGoups = this.currentTemplate.match(this.digitsAfterDot);
        var digits = 0;
        if (digitGoups[1] != null) {
            digits = digitGoups[1].length;
        }
        // parse the result of the exponential number and put it inot major and remainder 
        exp = this.value.toExponential(digits).match(this.exponentialRegExp);
        // groups:
        // [0]: whole number, e.g 7.747473452346e+5
        // [1]: digits before dot
        // [2]: digits after dot
        // [3]: sign of exponent
        // [4]: exponent  
        nMajor = (exp[1] != null) ? parseInt(exp[1], 10) : 0;
        nRemainder = (exp[2] != null) ? parseInt(exp[2], 10) : 0;
        exponentialSign = (exp[3] != null) ? exp[3] : "";
        strExponent = (exp[4] != null) ? exp[4] : "0";
        if ((this.exponentialSignTemplate != "+") && (exponentialSign == "+")) {
            // wipe out the "+"
            exponentialSign = "";
        }
    }
    var strMajor = "" + nMajor;
    var strRemainder = "" + nRemainder;
    // count digits. If we have more digits in the string than in the number, we must append 
    // leading zeros afterwards
    if (!isNumeric) {
        var mLength = this.valueAsString.split(".")[0].length;
        while ((mLength > strMajor.length && !this.isNegative) || ( mLength > (strMajor.length + 1) && this.isNegative)) {
            strMajor = "0" + strMajor;
        }
    }
    // do the remainder first
    var templateRemainder = "";
    var templateMajor = "";
    var hasFraction = false;
    if (firstDot > -1) {
        // format the remainder
        var templateRemainder = this.currentTemplate.substr(firstDot + 1);
        if (templateRemainder != "") {
            if ((this.exponentialDigits == 0) && (nRemainder != 0)) {
                // cut of "0."
                strRemainder = strRemainder.substr(2);
            }
            templateRemainder = templateRemainder.replace(this.dotsRegExp, "");
            // extend the remainder, if value is 0, but there should be multiple digits after the comma
            if (strRemainder == "0" && templateRemainder.length > strRemainder.length) {
                for (var i = 1; i < templateRemainder.length; i++) {
                    strRemainder += "0";
                }
            }
            this.strOutRemainder = this._formatNumber(templateRemainder, strRemainder, true /*isFraction*/ );
            // prepare for the integer part 
            templateMajor = this.currentTemplate.substr(0, firstDot);
        }
    }
    else {
        // prepare for the integer part, complete template is used for it 
        templateMajor = this.currentTemplate;
    }
    // now do the integer part, either with or without grouping
    this.strOutMajor = this._formatNumber(templateMajor, strMajor);
    ///////////////////////////////
    // format result
    ///////////////////////////////
    if ((firstDot > -1) && (this.strOutRemainder != "")) {
        this.formattedNumber = this.strOutMajor;
        this.formattedNumber += this.strOutRemainder;
    } else {
        this.formattedNumber = this.strOutMajor;
    }
    // Adjust Grouping
    // Adjusting separators for thousands for numeric values.
    // If there are no digits before the first comma, it is removed.
    // If a group of digits before the first comma consists of more than three symbols,
    // additional commas are inserted.
    var val = this.formattedNumber;
    var firstDigits = val.match(new RegExp("(\\"+this.GROUPING_DELIMITIER+")+(\\d)+"));
    if (firstDigits) {
        var pos = val.indexOf(firstDigits[0]);
        var begining = val.substring(0, pos);
        var ending = val.substring(pos, val.length);
        begining = begining.reverse().replace(/(\d{3})/g, "$1"+this.GROUPING_DELIMITIER).reverse().replace(new RegExp("^(-)?"+this.GROUPING_DELIMITIER,""), "$1");

        val = begining + ending;
    }
    //removing first decimal separators if there are no digits in the beginning
    firstDigits = val.match(new RegExp("(\\d)+"));
    if (firstDigits) {
        var pos = val.indexOf(firstDigits[0]);
        var begining = val.substring(0, pos);
        var ending = val.substring(pos, val.length);
        begining = begining.replace(new RegExp("\\"+this.GROUPING_DELIMITIER,"g"), "");
        val = begining + ending;
    }
    this.formattedNumber = val;
    // if value is negative, add minus on the left of first digit
    // but only if there is no special template for negative numbers, 
    // which must contain the minus then. 
    if (this.isNegative && (this.templates[1] == null)) {
        this.formattedNumber = this.formattedNumber.replace(this.leftmostDigitRegExp, this.MINUS_SIGN + "$1");
    }
    // format scientific, if required
    if (this.exponentialDigits > 0) {
        var expDigits = this._formatNumber(this.exponentialDigitsTemplate, strExponent, false /*isFraction*/ );
        this.formattedNumber = this.formattedNumber.replace(this.EXPONENTIAL_CHAR, this.EXPONENTIAL_CHAR + exponentialSign + expDigits);
    }
    return this.formattedNumber;
};

NumberFormatter.prototype._processSections = function () {
    // split into the various templates and choose the appropriate.
    this.templates = this.templateString.split(this.SECTION_SEPARATOR);
    var result;
    if (this.isNegative) {
        result = (this.templates[1] != null ? this.templates[1] : this.templates[0]);
    }
    else if (this.value == 0) {
        result = (this.templates[2] != null ? this.templates[2] : this.templates[0]);
    }
    else {
        result = this.templates[0];
    }
    this.currentTemplate = result;
    return result;
};

NumberFormatter.prototype._groupBy3 = function ( /*String*/ value) {
    var output = [];
    var count = 0;
    var c = "";
    var isDigit = new RegExp(/\d/);
    for (var i = value.length; i > 0; i--) {
        c = value.charAt(i - 1);
        output.push(c);
        if (c.match(isDigit)) {
            count++;
        }
        // add commas after 3 digits, but not if there are no digits left
        if (count == 3 && value.substr(0, i - 1).match(isDigit)) {
            output.push(this.GROUPING_DELIMITIER);
            count = 0;
        }
    }
    if ((output.length > 0) && (output[output.length - 1] == this.GROUPING_DELIMITIER)) {
        output.pop(); // do not start with a comma
    }
    output = output.reverse().join("");
    return output;
};

NumberFormatter.prototype._formatNumber = function ( /*String*/ template, /*String*/ value, /*bool*/ isFraction) {
    var output = [];
    var tmpOut = [];
    var valueChars = value.split("");
    var templateChars = template.split("");
    var hasFraction = false;
    var appendTmp = function() {
        output = output.concat(tmpOut);
        tmpOut = [];
    }
    if (!isFraction) {
        // numbers > 0 are processed from right to left
        valueChars.reverse();
        templateChars.reverse();
    }
    while (templateChars.length > 0) {
        var strChar = valueChars.length > 0 ? valueChars[0] : null;
        var tmplChar = templateChars[0];
        switch (tmplChar) {
        case this.ZERO_PLACEHOLDER:
            // '0'
            appendTmp();
            if (strChar == null) {
                output.push("0");
            }
            else {
                output.push(strChar);
            }
            if(isFraction)
              hasFraction = true;
            valueChars.shift();
            break;
        case this.DIGIT_PLACEHOLDER:
            // '#'
            if (strChar != null) {
                if (strChar != '0' || !isFraction) {
                    appendTmp();
                    output.push(strChar);
                    hasFraction = isFraction;
                } else {
                    // put to buffer temporarily
                    tmpOut.push(strChar);
                }
                valueChars.shift();
                if (!isFraction) {
                    // Check if this is the last Number placeholder
                    // if no, but there are more digits, this one is multiplied
                    var rest = templateChars.slice(1).join("");
                    if (rest.search(this.anyDigitRegExp) == -1) {
                        // this is the last placeholder, if there are more numbers in value
                        // reuse this placeholder
                        if (valueChars.length > 0) {
                            templateChars.unshift(this.DIGIT_PLACEHOLDER);
                        }
                    }
                }
            }
            break;
        case this.PADDING_PLACEHOLDER:
            // "?"
            if (strChar != null) {
                output.push(strChar);
                // Check if this is the last Number placeholder
                // if no, but there are more digits, this one is multiplied
                var rest = templateChars.slice(1).join("");
                if (rest.search(this.anyDigitRegExp) == -1) {
                    // this is the last placeholder, if there are more numbers in value
                    // reuse this placeholder
                    if (valueChars.length > 1) {
                        templateChars.unshift(this.DIGIT_PLACEHOLDER);
                    }
                }
            }
            else {
                output.push(" ");
            }
            valueChars.shift();
            break;
        default:
            // copy char to output
            output.push(tmplChar);
            // no shift here: valueChars.shift();
        }
        templateChars.shift();
    }
    if (!isFraction) {
        output.reverse();
    }
    var result = output.join("");
    if (hasFraction)
        result = this.DECIMAL_INDICATOR + result;
    return result;
};

String.prototype.reverse = function () {
    var res = "";
    for (var i = this.length; i > 0; --i) {
        res += this.charAt(i - 1);
    }
    return res;
}
