var logging_instrumentation_text = "Logging instrumentation data to: ";
var stopLogging_button_value = "Stop Logging";
var startLogging_button_value = "Start Logging";
var ok_button_value = "OK";
var loading_button_value = "Loading";
var error_label = "ERROR:";

dojo.require("dojox.uuid.Uuid");
dojo.require("dojox.uuid.generateRandomUuid");
dojo.require("dojox.collections.Dictionary");
dojo.require("dojox.collections._base");

Instrumentation.prototype = {

    _showOrHideDescendants : function (e, arr) {
        var thisObj = arr[0];
        var link = arr[1];
        var row = arr[2];

        var id = row.id;
        if (thisObj.currentData[id]['expanded']) {
            thisObj.currentData[id]['expanded'] = false;
            link.setAttribute('src', thisObj.imagesPrefix + "collapsed.gif");
        } else {
            thisObj.currentData[id]['expanded'] = true;
            link.setAttribute('src', thisObj.imagesPrefix + "expanded.gif");
        }

        thisObj._adjustRowVisibility();
    },

    setAutoRefresh : function (select) {
        var time = select.options[select.selectedIndex].value;
        if (this.runningTimer) {
            window.clearInterval(this.runningTimer);
            this.runningTimer = null;
        }
        if (time > 0) {
            time = time * 1000;
            window.setInterval(this.varName + ".refresh()", time);
        }
    },

    init : function () {
        dojo.connect(document, 'onkeydown', this, function(event) {
            this._checkForRefresh(event, this);
        });
        this.refresh();
    },

    _checkForRefresh : function (e, thisObj) {
        if (e.keyCode == dojo.keys.ENTER) { // ENTER
            thisObj.refresh();
            dojo.stopEvent(e);
        }
    },

    refresh : function () {
        this._getInstrumentation('instrumentation/refresh');
    },

    reset : function () {
        this.currentData = {};
        this._getInstrumentation('instrumentation/reset');
    },

    toggleLogging : function () {
        if (this.isLogging) {
            this._getInstrumentation('instrumentation/stopLogging');
        } else {
            this._getInstrumentation('instrumentation/startLogging');
        }
    },

    _getInstrumentation : function (url, additionalArgs) {
        this._markButtonsLoading();
        var callback = {
            success : this._ajaxInstrumentationSuccess,
            failure : this._ajaxInstrumentationFailure,
            argument : additionalArgs,
            scope : this
        }
        dojo.xhrGet({
            url: url + (this.userVisibleOnly?"?userVisibleOnly=true":"?userVisibleOnly=false"),
            handleAs: "xml",
            load: dojo.hitch(this, function(data) {
                this._ajaxInstrumentationSuccess(data);
            }),
            error: dojo.hitch(this, function(error) {
                this._ajaxInstrumentationFailure(error);
            }),
            headers: {"If-Modified-Since": "Sat, 1 Jan 2000 00:00:00 GMT"}
        });
    },

    _ajaxInstrumentationSuccess : function (obj) {
        this._markButtonsLoadingFinished();
        this.isLogging = (obj.documentElement.getAttribute('isLogging') == 'true');
        if (this.isLogging) {
            this._setLoggingButtons(true, obj.documentElement.getElementsByTagName('logFilePath'));
        } else {
            this._setLoggingButtons(false);
        }
        var insts = obj.documentElement.getElementsByTagName('inst');
        var tbody = document.getElementById(this.tbodyId);
        var rows = tbody.getElementsByTagName('tr');
        var rowsToRemove = rows.length - insts.length;
        var newData = {};
        var previousDepth = 0;
        for (var i = 0; i < insts.length; ++i) {
            var row;
            if (i >= rows.length) {
                row = this._addNewRow(tbody);
            } else {
                row = rows[i];
            }
            var cells = rows[i].getElementsByTagName('td');

            var id = insts[i].getAttribute('id');
            newData[id] = new Object();
            newData[id]['id'] = id;
            if (this.currentData[id] && !this.currentData[id]['expanded']) {
                newData[id]['expanded'] = false;
            } else {
                newData[id]['expanded'] = true;
            }
            row.id = id;

            var depth = insts[i].getAttribute('depth');

            if (i > 0) {
                this._addExpandCollapseControl(rows[i-1], previousDepth, depth, newData[rows[i-1].id]['expanded']);
            }
            previousDepth = depth;

            newData[id]['depth'] = depth;
            var name = "";
            for (var j = 0; j < depth * 4; ++j) {
                name += "&nbsp;";
            }
            name += insts[i].getAttribute('name');
            cells[0].getElementsByTagName('span')[0].innerHTML = name;
            var desc = insts[i].getElementsByTagName('description');
            if (desc.length > 0) {
                var doc = this._textContent(desc[0]);
                cells[0].setAttribute('title', doc);
                newData[id]['doc'] = doc;
            } else {
                cells[0].setAttribute('title', insts[i].getAttribute('name'));
                newData[id]['doc'] = insts[i].getAttribute('name');
            }

            var type = insts[i].getAttribute('type');
            if ('Period' == type) {
                this._setCellValue(insts[i], cells[1], newData[id], this.currentData[id], 'count');
                this._setCellValue(insts[i], cells[2], newData[id], this.currentData[id], 'inProcess');
                this._setCellValue(insts[i], cells[3], newData[id], this.currentData[id], 'averageDuration');
                this._setCellValue(insts[i], cells[4], newData[id], this.currentData[id], 'movingAverageDuration');
                this._setCellValue(insts[i], cells[5], newData[id], this.currentData[id], 'total');
            } else if ('Point' == type) {
                this._setCellValue(insts[i], cells[1], newData[id], this.currentData[id], 'count');
                this._blankCells(cells, 2);
            } else if ('Value' == type) {
                this._setCellValue(insts[i], cells[1], newData[id], this.currentData[id], 'value');
                this._blankCells(cells, 2);
                this._setCellValue(insts[i], cells[3], newData[id], this.currentData[id], 'averageDuration');
                this._setCellValue(insts[i], cells[4], newData[id], this.currentData[id], 'movingAverageDuration');
                //this._setCellValue(insts[i], cells[5], newData[id], this.currentData[id], 'total');
            } else {
                this._blankCells(cells, 1);
            }

        }
        if (rowsToRemove > 0) {
            for (var i = rows.length - 1; i >= insts.length; --i) {
                var cells = rows[i].getElementsByTagName('td');

                var theCell = cells[0].getElementsByTagName('span')[0];
                var entry = this.eventMap.entry(theCell.id).valueOf();
                this.eventMap.remove(theCell.id);
                dojo.disconnect(entry);

                theCell = cells[0].getElementsByTagName('img')[0];
                entry = this.eventMap.entry(theCell.id).valueOf();
                this.eventMap.remove(theCell.id);
                dojo.disconnect(entry);

                tbody.removeChild(rows[i]);
            }
        }
        this.currentData = newData;
        this._adjustRowVisibility();
    },

    _addNewRow : function (tbody) {
        var row = document.createElement('tr');
        tbody.appendChild(row);
        for (var j = 0; j < 6; ++j) {
            var cell = document.createElement('td');
            if (j == 0) {
                var link = document.createElement('img');
                link.setAttribute('src', this.imagesPrefix + "empty.gif");
                if(!link.id) {
                    link.id = new dojox.uuid.Uuid(dojox.uuid.generateRandomUuid());
                }
                cell.appendChild(link);
                var linkEventHandle = dojo.connect(link, 'onclick', this, function(event){
                    this._showDocumentation(event, [this,row])
                });
                this.eventMap.add(link.id, linkEventHandle);

                var span = document.createElement('span');
                if(!span.id) {
                    span.id = new dojox.uuid.Uuid(dojox.uuid.generateRandomUuid());
                }
                cell.appendChild(span);
                var spanEventHandle = dojo.connect(span, 'onclick', this, function(event){
                    this._showDocumentation(event, [this,row])
                });
                this.eventMap.add(span.id, spanEventHandle);
                span.appendChild(document.createTextNode(" "));
            } else {
                cell.setAttribute('align', 'right');
                cell.appendChild(document.createTextNode(" "));
            }

            row.appendChild(cell);
        }
        return row;
    },

    _addExpandCollapseControl : function (previousRow, previousDepth, depth, expanded) {
        // IE returns resolved value for src attribute, Firefox returns value that was set. I'd like to
        // use endsWith but javascript doesn't have one, hence use of indexOf
        var link = previousRow.getElementsByTagName('td')[0].getElementsByTagName('img')[0];
        if (depth > previousDepth) {
            if (link.getAttribute('src').indexOf('images/empty.gif') > -1) {
                if(!link.id) {
                    link.id = new dojox.uuid.Uuid(dojox.uuid.generateRandomUuid());
                }
                var eventHandle = dojo.connect(link, 'onclick', this, function(event) {
                    this._showOrHideDescendants(event, [this, link, previousRow]);
                });
                this.eventMap.add(link.id, eventHandle);
            }
            if (expanded) {
                link.setAttribute('src', this.imagesPrefix + "expanded.gif");
            } else {
                link.setAttribute('src', this.imagesPrefix + "collapsed.gif");
            }
        } else {
            if (link.getAttribute('src').indexOf('images/empty.gif') == -1) {
                link.setAttribute('src', this.imagesPrefix + "empty.gif");

                var entry = this.eventMap.entry(link.id).valueOf();
                this.eventMap.remove(link.id);
                dojo.disconnect(entry);
            }
        }
    },

    _setCellValue : function (inst, cell, newData, currentData, elementName) {
        var value = this._textContent(inst.getElementsByTagName(elementName)[0]);
        newData[elementName] = value;
        cell.innerHTML = value;
        if (currentData && currentData[elementName] != value) {
            this._addClass(cell, 'value-changed');
        } else {
            this._removeClass(cell, 'value-changed');
        }
    },

    _addClass : function (el, className) {
        var classes = this._getClasses(el);
        for (var i = 0; i < classes.length; ++i) {
            if (classes[i] == className) return;
        }
        classes.push(className);
        el.setAttribute('class', classes.join(' '));
    },

    _removeClass : function (el, className) {
        var classes = this._getClasses(el);
        for (var i = 0; i < classes.length; ++i) {
            if (classes[i] == className) {
                classes.splice(i, 1);
                el.setAttribute('class', classes.join(' '));
                return;
            }
        }
    },

    _getClasses : function (el) {
        var classAttr = el.getAttribute('class');
        if (classAttr) {
            return classAttr.split(/\s+/);
        } else {
            return [];
        }
    },

    _adjustRowVisibility : function () {
        var hideDepth = -1;
        var tbody = document.getElementById(this.tbodyId);
        var rows = tbody.getElementsByTagName('tr');
        for (var i = 0; i < rows.length; ++i) {
            var id = rows[i].id;
            var depth = this.currentData[id]['depth'];

            var display = dojo.style(rows[i], 'display');
            if (hideDepth != -1 && depth > hideDepth) {
                if (!display || display != 'none') dojo.style(rows[i], 'display', 'none');
            } else {
                // should be table-row but IE doesn't support that. IE does support block but firefox doesn't so use
                // empty string which should make both fall back to their defaults
                if (!display || display != '') dojo.style(rows[i], 'display', '');
                if (this.currentData[id]['expanded']) {
                    hideDepth = -1;
                } else {
                    hideDepth = depth;
                }
            }
        }
    },

    _ajaxInstrumentationFailure : function (obj) {
        var text = obj.responseText;
        this._createErrorMsg(text);
    },
    
    _createErrorMsg: function(msg) {
		var errorMetaTag = this._getXMLElementText(this._getXMLElementText(msg,"tw:xmlData"), "error");
		if (errorMetaTag) {
			var message = this._getXMLElementText(errorMetaTag, "message");
			if (message != null) {
				// Show message instead
				msg = message;
			}
		}
		if (!msg) {
			msg = "An unknown error has occured.";
		}
    	if (this.errorId) {
    		var e = document.getElementById(this.errorId);
    		if (e) {
    			while (e.hasChildNodes()) {
    				e.removeChild(e.childNodes[0]);
    			}
    			
    			e.appendChild(document.createElement('br'));
    			var span = document.createElement('span');
    			var text = document.createTextNode(error_label + " " + msg);
    			span.appendChild(text);
    			e.appendChild(span);
    			e.style.display = '';
    		}
    	}
    },
    

	_getXMLElementText : function(msg, tagName) {
		if (typeof msg !== "string") return;
		var startIndex = msg.indexOf("<" + tagName + ">");
		if (startIndex === -1) return null;
		startIndex = startIndex + tagName.length + 2;
		var rest = msg.substring(startIndex);
		var endIndex = rest.indexOf("</" + tagName + ">");
		if (endIndex === -1) return;
		return rest.substring(0, endIndex);
	},
    
    _blankCells : function (cells, startingIndex) {
        for (var i = startingIndex; i < cells.length; ++i) {
            cells[i].innerHTML = '&nbsp;';
            this._removeClass(cells[i], 'value-changed');
        }
    },

    _markButtonsLoading : function () {
        for (var i = 0; i < this.refreshIds.length; ++i) {
            var e = dojo.byId(this.refreshIds[i]);
            this.originalTexts[this.refreshIds[i]] = this._textContent(e);
            e.innerHTML = loading_button_value;
        }
        for (var i = 0; i < this.resetIds.length; ++i) {
            var e = dojo.byId(this.resetIds[i]);
            this.originalTexts[this.resetIds[i]] = this._textContent(e);
            e.innerHTML = loading_button_value;
        }
        for (var i = 0; i < this.loggingIds.length; ++i) {
            var e = dojo.byId(this.loggingIds[i]);
            this.originalTexts[this.loggingIds[i]] = this._textContent(e);
            e.innerHTML = loading_button_value;
        }
    },

    _markButtonsLoadingFinished : function () {
        for (var i = 0; i < this.refreshIds.length; ++i) {
            var e = dojo.byId(this.refreshIds[i]);
            e.innerHTML = this.originalTexts[this.refreshIds[i]];
        }
        for (var i = 0; i < this.resetIds.length; ++i) {
            var e = dojo.byId(this.resetIds[i]);
            e.innerHTML = this.originalTexts[this.resetIds[i]];
        }
        for (var i = 0; i < this.loggingIds.length; ++i) {
            var e = dojo.byId(this.loggingIds[i]);
            e.innerHTML = this.originalTexts[this.loggingIds[i]];
        }
    },

    _setLoggingButtons : function (isLogging, logFilePathElements) {
        for (var i = 0; i < this.loggingIds.length; ++i) {
            var e = dojo.byId(this.loggingIds[i]);
            e.innerHTML = (isLogging?stopLogging_button_value:startLogging_button_value);
        }
        var e = dojo.byId(this.logFileId);
        if (isLogging && logFilePathElements) {
            e.innerHTML = logging_instrumentation_text + "\"" + this._textContent(logFilePathElements[0]) + "\"";
        } else {
            e.innerHTML = "";
        }
    },

    _showDocumentation : function (e, arr) {
        var thisObj = arr[0];
        var row = arr[1];
        var doc = thisObj.currentData[row.id]['doc'];
        var docpopup = document.getElementById(thisObj.documentationId);
        dojo.style(docpopup, 'display', 'block');
        var x = e.event.pageX;
        var y = e.event.pageY;
        dojo.style(docpopup, {left: x+"px", top: y+"px"});
        var children = docpopup.childNodes;
        for (var i = children.length -1; i >= 0; --i) {
            if (children[i].nodeName == 'button') {
                var child = children[i];
                entry = this.eventMap.entry(child.id).valueOf();
                this.eventMap.remove(child.id);
                dojo.disconnect(entry);
            }
            docpopup.removeChild(children[i]);
        }
        docpopup.appendChild(document.createTextNode(doc));
        docpopup.appendChild(document.createElement('br'));
        var button = document.createElement('button');
        button.appendChild(document.createTextNode(ok_button_value));
        var eventHandle = dojo.connect(button, 'onclick', this, function(event){
            thisObj._hideDocumentation(event, docpopup)
        });
        this.eventMap.add(span.id, eventHandle);
        docpopup.appendChild(button);

        dojo.stopEvent(e);
    },

    _hideDocumentation : function (e, docpopup) {
        dojo.style(docpopup, 'display', 'none');
        dojo.stopEvent(e);
    },

    _textContent : function (elem) {
        if (elem.textContent) {
            return elem.textContent;
        } else {
            var text = "";
            for (var i = 0; i < elem.childNodes.length; ++i) {
                text += elem.childNodes[i].nodeValue;
            }
        }
        return text;
    }

}
function Instrumentation (varName, imagesPrefix, tbodyId, documentationId, logFileId, refreshIds, resetIds, loggingIds, userVisibleOnly, errorId) {
    this.varName = varName;
    this.imagesPrefix = imagesPrefix;
    this.tbodyId = tbodyId;
    this.documentationId = documentationId;
    this.logFileId = logFileId;
    this.refreshIds = refreshIds;
    this.resetIds = resetIds;
    this.loggingIds = loggingIds;
    this.userVisibleOnly = userVisibleOnly;
    this.originalTexts = new Array();
    this.currentData = {};
    this.eventMap = new dojox.collections.Dictionary();
    this.errorId = errorId;
}
