dojo.provide("lombardi.CheckBoxTree");
dojo.require("dijit.Tree");

/**
 * CheckBoxTreeNode implementation showing checkbox for tree nodes having 'checkbox' attribute.
 * Nodes with 'checkbox' attribute undefined will not have checkbox in front of them.
 */
dojo.declare( "lombardi._CheckBoxTreeNode", dijit._TreeNode,
{
    checkbox: null,

    _createCheckbox: function() {
        var itemState = this.tree.model.getCheckboxState(this.item);
        // Show checkbox only for items having 'checkbox' attribute defined.
        if (itemState !== undefined) {
            this.checkbox = dojo.doc.createElement('input');
            this.checkbox.type = 'checkbox';
            this.checkbox.checked = itemState;
			this.checkbox.value = this.item.value;
            // Put checkbox before node label.
            dojo.place(this.checkbox, this.expandoNode, 'after');
        }
    },

    postCreate: function() {
        this._createCheckbox();
        this.inherited(arguments);
    }
});

/**
 * CheckBoxTree
 * Provides 'onNodeChecked', 'onNodeUnchecked' events.
 */
dojo.declare( "lombardi.CheckBoxTree", dijit.Tree,
{
	readOnly: false,

    onNodeChecked: function(/*dojo.data.Item*/ storeItem, /*treeNode*/ nodeWidget) {
    },

    onNodeUnchecked: function(/*dojo.data.Item*/ storeItem, /*treeNode*/ nodeWidget) {
    },

    _onClick: function(/*Event*/ event) {

        var domElement = event.target;
        if (domElement.nodeName != 'INPUT') return this.inherited(arguments);

        var widget = dijit.getEnclosingWidget(domElement);
        if (widget.checkbox.checked) {
            this.onNodeChecked(widget.item, widget);
        } else {
            this.onNodeUnchecked(widget.item, widget);
        }
        this._setCheckboxState(widget.item, widget.checkbox.checked);
    },

    _createTreeNode: function(args) {
        return new lombardi._CheckBoxTreeNode(args);
    },

	_setCheckboxState: function( storeItem,  newState ) {
		if (storeItem) {
			this.tree.model.updateCheckbox(storeItem, newState);
		}
	},
	 
	_setNode: function(/*_CheckBoxTreeNode*/ node, /*boolean*/ checked) {		
		if(node.checkbox!=null) {
			node.checkbox.checked = checked;
			this._setCheckboxState(node.item, node.checkbox.checked);
		}
		
		for(var i=0; i<node.getChildren().length; i++) {
			this._setNode(node.getChildren()[i], checked);
		}
	 },
	 
	 setAll: function(/*boolean*/ checked) {
		if(!this.readOnly) {
			if(checked) { this.expandAll(); }
			this._setNode(this.rootNode, checked);
		}
	 },
	 
	setReadOnly: function(/*boolean*/ readOnly) {
		var setReadOnly = function(/*_CheckBoxTreeNode*/ node, /*boolean*/ readOnly) {
			if(node.checkbox!=null) {
				node.checkbox.disabled = readOnly;
			}

			for(var i=0; i<node.getChildren().length; i++) {
				setReadOnly(node.getChildren()[i], readOnly);
			}
		}
	
		this.readOnly = readOnly;
		setReadOnly(this.rootNode, readOnly);
	},
	 
	_setCheckboxNames: function(/*_CheckBoxTreeNode*/ node, /*String*/ name) {
		if(node.checkbox!=null) {			
			node.checkbox.name = name;
		}
		
		for(var i=0; i<node.getChildren().length; i++) {
			this._setCheckboxNames(node.getChildren()[i], name);
		}
	},
	 
	setCheckboxNames: function(/*String*/ name) {
		this._setCheckboxNames(this.rootNode, name);
	},
	 
	get: function(/*String*/ id) {
		var tree = this;
		var _get = function(node) {
			if(node.item!=null && node.item.value==id) {
				return node;
			}
			else {
				for(var i=0; i<node.getChildren().length; i++) {
					var result = _get(node.getChildren()[i]);
					
					if(result!=null) {
						return result;
					}
				}
			}
		}
				
		return _get(this.rootNode);
	},
	 
	setById: function(/*String*/ id, /*boolean*/ checked) {
		var node = this.get(id);
				
		if(node!=null && node.checkbox!=null) {
			node.checkbox.checked = checked;
			this._setCheckboxState(node.item, node.checkbox.checked);
		}
	},	 
	 
	expandAll: function() {
		var tree = this;
		var expandNode = function(node) {
			tree._expandNode(node);
			dojo.forEach(node.getChildren(), expandNode);
		};
		
		expandNode(this.rootNode);
	},
	
	postCreate: function() {        
        this.inherited(arguments);
		this.expandAll();
    }
});

/**
 * TreeStoreModel stores CheckBoxTree model.
 * CheckBox state of node is stored in 'checkbox' property.
 */
dojo.declare("lombardi.CheckBoxStoreModel", dijit.tree.TreeStoreModel,
{
    checkboxAttr: "checkbox",
    checkboxState: false,

    updateCheckbox: function(/*dojo.data.Item*/ storeItem, /*Boolean*/ newState) {
        var /*Boolean*/ oldValue = this.store.getValue(storeItem, this.checkboxAttr);
        if (oldValue != newState && oldValue !== undefined) {
            this.store.setValue(storeItem,  this.checkboxAttr, newState);
            this.onCheckboxChange(storeItem);
        }

    },

    getCheckboxState: function(/*dojo.data.Item*/ storeItem) {
        return this.store.getValue(storeItem, this.checkboxAttr);
    },

    /**
     * Public event is called if any store item has been changed.
     * @param storeItem that has been changed.
     */
    onCheckboxChange: function(/*dojo.data.Item*/ storeItem) {
    }
});
